c***********************************************************************
c Subroutines and Functions describing the running of the strong coupling 
c quark masses. Using the results presented in Chetyrkin, Kuhn and 
c Steinhauser (hep-ph/0004189) and references therein.
c***********************************************************************

      Subroutine StrSetup(alphaS_0,mu_0,nf,m_q,t_q,m_l,mu_l)

c Subroutine that sets up the various common blocks needed when calling
c the various functions given below.

      Implicit NONE

c Inputs

c The strong coupling alphaS at the scale mu_0
      Double Precision alphaS_0,mu_0
c The charm, bottom and top masses
      Double Precision m_q(3)
c The down,up, and strange masses and the corresponding renormalization
c scale
      Double Precision m_l(3),mu_l(3)
c The number of flavours, and whether the masses are SI or pole masses
      Integer nf,t_q(3)

c Local
      Integer i,j,k,itmp
      Double Precision m_si(3),m_po(3)

c Functions
      Double Precision LambdaS,LamUp,LamDown
      Double Precision SI2Pole, Pole2SI

c Common blocks
      Double Precision StrLambda(4,6)
      Common/STRlams/StrLambda

      Integer NFq(6)
      Double Precision STRmass(6),STRmus(6),STRpole(6)
      Common/STRmasses/STRmass,STRmus,STRpole,NFq

c Initialise arrays.
      Do i=1,4
         Do j=1,6
            StrLambda(i,j)=0.D0
         End Do
      End Do

      Do i=1,6
         STRmass(i)=0.D0
         STRmus(i)=0.D0
         STRpole(i)=0.D0
      End Do

      Do i=1,3
         STRmass(i)=m_l(i)
         STRmus(i)=mu_l(i)
c Heavy quarks c,b,t
         STRmus(i+3)=m_q(i)
         STRpole(i+3)=m_q(i)
         STRmass(i+3)=m_q(i)
         m_si(i)=m_q(i)
         m_po(i)=m_q(i)
      End Do

      Do i=1,6
         itmp=0
         Do j=1,6
            If (STRmus(i).ge.STRmass(j)) Then
               itmp=itmp+1
            End If
         End Do
         NFq(i)=itmp
      End Do

c Find the value of lambda for a given number of loops

      Do i=1,10
c Find the values of Lambda taking into account of threshold effects etc
         If (nf.eq.4) Then
            Do j=1,4
               StrLambda(j,4)=LambdaS(alphaS_0,mu_0,nf,j)
               StrLambda(j,5)=LamUp(StrLambda(j,4),m_si(2),5,j)
               StrLambda(j,6)=LamUp(StrLambda(j,5),m_si(3),6,j)
               StrLambda(j,3)=LamDown(StrLambda(j,4),m_si(1),3,j)
            End Do
         Else If (nf.eq.5) Then
            Do j=1,4
               StrLambda(j,5)=LambdaS(alphaS_0,mu_0,nf,j)
               StrLambda(j,6)=LamUp(StrLambda(j,5),m_si(3),6,j)
               StrLambda(j,4)=LamDown(StrLambda(j,5),m_si(2),4,j)
               StrLambda(j,3)=LamDown(StrLambda(j,4),m_si(1),3,j)
            End Do
         Else
            Do j=1,4
               StrLambda(j,6)=LambdaS(alphaS_0,mu_0,nf,j)
               StrLambda(j,5)=LamDown(StrLambda(j,6),m_si(3),5,j)
               StrLambda(j,4)=LamDown(StrLambda(j,5),m_si(2),4,j)
               StrLambda(j,3)=LamDown(StrLambda(j,4),m_si(1),3,j)
            End Do
         End If
c Setup the initial values of the quark masses used in the function
c rmass.

c Light quarks d,u,s
         Do j=1,3
            STRmass(j)=m_l(j)
            STRmus(j)=mu_l(j)
         End Do

         Do j=1,3
c Heavy quarks c,b,t
            If (t_q(j).eq.1) Then
               m_si(j)=Pole2SI(m_po(j),3+j,3+j-1,4)
            Else
               m_po(j)=SI2Pole(m_si(j),3+j,3+j-1,4)
            End If
            STRmass(j+3)=m_si(j)
            STRmus(j+3)=m_si(j)
            STRpole(j+3)=m_po(j)
         End Do

         Do j=1,6
            itmp=0
            Do k=1,6
               If (STRmus(j).ge.STRmass(k)) Then
                  itmp=itmp+1
               End If
            End Do
            NFq(j)=itmp
         End Do
      End Do

      Return
      End

c***********************************************************************

      Double Precision Function LambdaS(alphaS_0,mu_0,nf,nl)

c Function that computes the strong coupling parameter LambdaS needed
c when computing the running of the strong coupling constant alphaS.
c To do this equation 4 of Chetyrkin, Kuhn and Steinhauser (CKS
c henceforth) is used.

      Implicit NONE

c Inputs

c     The number of active flavours and the required accuracy (1-4).
      Integer nf,nl
c The input value of alphaS and the corresponding scale
      Double Precision alphaS_0, mu_0

c Local
      Integer i
      Double Precision etal(4)
      Double Precision tmp,a,b1,b2,b3

c Functions
      Double Precision beta0,beta1,beta2,beta3

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nl
         etal(i)=1.D0
      End Do

      a=alphaS_0/PI
      b1=beta1(nf)/beta0(nf)
      b2=beta2(nf)/beta0(nf)
      b3=beta3(nf)/beta0(nf)

      tmp=1.D0/a*etal(1)
     &     +b1*LOG(a)*etal(2)
     &     +(b2-b1**2)*a*etal(3)
     &     +(b3/2.D0-b1*b2+b1**3/2.D0)*a**2*etal(4)

      tmp=tmp*1.D0/beta0(nf)+b1/beta0(nf)*LOG(beta0(nf))*etal(2)

      LambdaS=mu_0*EXP(-tmp/2.D0)

      Return
      End

c***********************************************************************

      Double Precision Function alphaStr(mu,nf,nl)

c Function that returns the strong coupling constant alphaStr at the
c scale mu in a theory with nf active flavours using formula (5) of
c CKS

      Implicit NONE

c Inputs

c The number of active flavours and loop order
      Integer nf,nl
c The desired scale
      Double Precision mu

c Local
      Integer i
      Double Precision etal(4)
      Double Precision L,b1,b2,b3,tmp

c Functions
      Double Precision beta0,beta1,beta2,beta3

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

c Common blocks
      Double Precision StrLambda(4,6)
      Common/STRlams/StrLambda

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nl
         etal(i)=1.D0
      End Do

      L=LOG(mu**2/StrLambda(nl,nf)**2)

      b1=beta1(nf)/beta0(nf)
      b2=beta2(nf)/beta0(nf)
      b3=beta3(nf)/beta0(nf)

      tmp=1.D0/(beta0(nf)*L)*etal(1)
     &     -b1*LOG(L)/((beta0(nf)*L)**2)*etal(2)
     &     +1.D0/((beta0(nf)*L)**3)
     &     *(b1**2*(LOG(L)**2-LOG(L)-1.D0)+b2)*etal(3)
     &     +1.D0/((beta0(nf)*L)**4)
     &     *(b1**3*(-LOG(L)**3+5.D0/2.D0*LOG(L)**2
     &     +2.D0*LOG(L)-1.D0/2.D0)-3.D0*b1*b2*LOG(L)+b3/2.D0)*etal(4)

      alphaStr=PI*tmp

      Return
      End

c***********************************************************************

      Double Precision Function MSrmass(mu,gen,nf,nl)

c Function that determines the running mass at a scale mu in a theory
c with nf active flavours of the quark gen where :
c gen=1 down
c gen=2 up
c gen=3 strange
c gen=4 charm
c gen=5 bottom
c gen=6 top
c Using equation 9 of CKS

      Implicit NONE

c Inputs

c The generation of the required quark, the number of flavours and the
c desired loop accuracy
      Integer gen,nf,nl

c The desired scale
      Double Precision mu

c Local
      Integer i,diff
      Double Precision m_0,a,a_0,mu_0

c Functions
      Double Precision c_MSrmass,alphaStr,MassUpPole,MassDownPole

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

c Common blocks
      Integer NFq(6)
      Double Precision STRmass(6),STRmus(6),STRpole(6)
      Common/STRmasses/STRmass,STRmus,STRpole,NFq

      m_0=STRmass(gen)
      mu_0=STRmus(gen)

c      If (NFq(gen).lt.nf.and.nf.gt.2) Then
c         diff=nf-NFq(gen)
c         Do i=1,diff
c            m_0=MassUpPole(m_0,STRmus(gen),STRpole(NFq(gen)+i),nf,nl)
c         End Do
c      Else If (NFq(gen).gt.nf.and.nf.gt.2) Then
c         diff=NFq(gen)-nf
c         Do i=1,diff
c            m_0=MassDownPole(m_0,STRmus(gen),STRpole(NFq(gen)-i+1),
c     &           nf,nl)
c         End Do
c      End If

c      a=alphaStr(mu,nf,nl)/PI
c      a_0=alphaStr(STRmus(gen),nf,nl)/PI

c      MSrmass=m_0*c_MSrmass(a,nf,nl)/c_MSrmass(a_0,nf,nl)

c      Write(*,*) nf, NFq(gen)

      If (NFq(gen).lt.nf.and.nf.gt.2) Then
         a=alphaStr(mu,NFq(gen),nl)/PI
         a_0=alphaStr(STRmus(gen),NFq(gen),nl)/PI
         m_0=m_0*c_MSrmass(a,NFq(gen),nl)/c_MSrmass(a_0,NFq(gen),nl)
         diff=nf-NFq(gen)
         Do i=1,diff
            m_0=MassUpPole(m_0,mu,STRpole(NFq(gen)+i),nf,nl)
         End Do
      Else If (NFq(gen).gt.nf.and.nf.gt.2) Then
         a=alphaStr(mu,nf,nl)/PI
         a_0=alphaStr(STRmus(gen),nf,nl)/PI
         diff=NFq(gen)-nf
         Do i=1,diff
            m_0=MassDownPole(m_0,STRmus(gen),STRpole(NFq(gen)-i+1),
     &           nf,nl)
         End Do
         m_0=m_0*c_MSrmass(a,nf,nl)/c_MSrmass(a_0,nf,nl)
      Else
         a=alphaStr(mu,nf,nl)/PI
         a_0=alphaStr(STRmus(gen),nf,nl)/PI
         m_0=m_0*c_MSrmass(a,nf,nl)/c_MSrmass(a_0,nf,nl)
      End If
      
      MSrmass=m_0

      Return
      End

c***********************************************************************

      Double Precision Function c_MSrmass(x,nf,nl)

c Equation 10 of CKS

      Implicit NONE
      
c Inputs
      Double Precision x

c The number of active flavours
      Integer nf,nl

c Local
      Integer i
      Double Precision etal(4)

c Functions      
      Double Precision gamma0,gamma1,gamma2,gamma3
      Double Precision beta0,beta1,beta2,beta3
      Double Precision c0,c1,c2,c3,b0,b1,b2,b3

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nl
         etal(i)=1.D0
      End Do

      c0=gamma0(nf)/beta0(nf)
      c1=gamma1(nf)/beta0(nf)
      c2=gamma2(nf)/beta0(nf)
      c3=gamma3(nf)/beta0(nf)

      b0=beta0(nf)/beta0(nf)
      b1=beta1(nf)/beta0(nf)
      b2=beta2(nf)/beta0(nf)
      b3=beta3(nf)/beta0(nf)

       c_MSrmass=x**c0
     &     *(1.D0*etal(1)
     &     +(c1-b1*c0)*x*etal(2)
     &     +1.D0/2.D0*((c1-b1*c0)**2+c2-b1*c1+b1**2*c0-b2*c0)
     &     *x**2*etal(3)
     &     +(1.D0/6.D0*(c1-b1*c0)**3
     &     +1.D0/2.D0*(c1-b1*c0)*(c2-b1*c1+b1**2*c0-b2*c0)
     &     +1.D0/3.D0*(c3-b1*c2+b1**2*c1-b2*c1-b1**3*c0
     &     +2.D0*b1*b2*c0-b3*c0))*x**3*etal(4))

      Return
      End

c***********************************************************************

      Double Precision Function IterSI(mass,gen,nf,nl)

c Function that iteratively finds the scale invariant mass m_h(m_h)

      Implicit NONE

c Inputs
      Double Precision mass
      Integer nf,gen,nl

c Local
      Integer i
      Double Precision tmp

c Functions
      Double Precision MSrmass
     
      tmp=mass

      Do i=1,20
         tmp=MSrmass(tmp,gen,nf,nl)
      End Do

      IterSI=tmp

      Return
      End

c***********************************************************************

      Double Precision Function Pole2MS(mu,mpole,nf,nl,nloop)

c Function that converts the polemass to the MS-bar running mass at the
c scale mu with nl lighter quarks in a theory with nf active flavours
c using equation 13 of CKS.

      Implicit NONE

c Inputs
      Double Precision mu, mpole
      Integer nf,nl,nloop

c Local
      Integer i
      Double Precision etal(4)
      Double Precision lum,a,delfac,zfac

c Functions 
      Double Precision alphaStr,STRdelta

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta2
      Parameter(zeta2=1.6449340668482264364724151666460255D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision zeta5
      Parameter(zeta5=1.0369277551433699263313654871823873D0)
      Double Precision a4
      Parameter(a4=0.517479061673899386330758161878231D0)

c Common blocks
      Integer NFq(6)
      Double Precision STRmass(6),STRmus(6),STRpole(6)
      Common/STRmasses/STRmass,STRmus,STRpole,NFq

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      a=alphaStr(mu,nf,nloop)/PI
      lum=LOG(mu**2/mpole**2)

      zfac=-9478333.D0/93312.D0+55.D0/162.D0*LOG(2.D0)**4
     &     +(-644201.D0/6480.D0+587.D0/27.D0*LOG(2.D0)
     &     +44.D0/27.D0*LOG(2.D0)**2)*zeta2
     &     -61.D0/27.D0*zeta3+3475.D0/432.D0*zeta4
     &     +1439.D0/72.D0*zeta2*zeta3
     &     -1975.D0/216.D0*zeta5+220.D0/27.D0*a4
     &     +nl*(246643.D0/23328.D0-1.D0/81.D0*LOG(2.D0)**4
     &     +(967.D0/108.D0+22.D0/27.D0*LOG(2.D0)
     &     -4.D0/27.D0*LOG(2.D0)**2)*zeta2
     &     +241.D0/72.D0*zeta3-305.D0/108.D0*zeta4-8.D0/27.D0*a4)
     &     +nl**2*(-2353.D0/23328.D0-13.D0/54.D0*zeta2-7.D0/54.D0*zeta3)
      
      delfac=0.D0

      Do i=1,nl
         delfac=delfac+STRdelta(STRpole(nl)/mpole)
      End Do

      Pole2MS=mpole*(1.D0
     &     +a*(-4.D0/3.D0+lum)*etal(2)
     &     +a**2*(-3019.D0/288.D0-2.D0*zeta2-2.D0/3.D0*zeta2*LOG(2.D0)
     &     +1.D0/6.D0*zeta3-445.D0/72.D0*lum-19.D0/24.D0*lum**2
     &     +(71.D0/144.D0+1.D0/3.D0*zeta2+13.D0/36.D0*lum
     &     +1.D0/12.D0*lum**2)*nl-4.D0/3.D0*delfac)*etal(3)
     &     +a**3*(zfac+(-165635.D0/2592.D0-25.D0/3.D0*zeta2
     &     -25.D0/9.D0*zeta2*LOG(2.D0)+55.D0/36.D0*zeta3)*lum
     &     -11779.D0/864.D0*lum**2-475.D0/432.D0*lum**3
     &     +nl*((10051.D0/1296.D0+37.D0/18.D0*zeta2
     &     +2.D0/9.D0*zeta2*LOG(2.D0)+7.D0/9.D0*zeta3)*lum
     &     +911.D0/432.D0*lum**2+11.D0/54.D0*lum**3)
     &     +nl**2*((-89.D0/648.D0-1.D0/9.D0*zeta2)*lum**2
     &     -13.D0/216.D0*lum**2-1.D0/108.D0*lum**3))*etal(4)
     &     )

      Return
      End

c***********************************************************************

      Double Precision Function Pole2SI(mpole,nf,nl,nloop)

c Function that converts the polemass to the MS-bar running mass at the
c scale mu with nl lighter quarks in a theory with nf active flavours
c using equation 13 of CKS.

      Implicit NONE

c Inputs
      Double Precision mpole
      Integer nf,nl,nloop

c Local
      Integer i
      Double Precision etal(4)
      Double Precision a,delfac,zfac

c Functions 
      Double Precision alphaStr,STRdelta

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta2
      Parameter(zeta2=1.6449340668482264364724151666460255D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision zeta5
      Parameter(zeta5=1.0369277551433699263313654871823873D0)
      Double Precision a4
      Parameter(a4=0.517479061673899386330758161878231D0)

c Common blocks
      Integer NFq(6)
      Double Precision STRmass(6),STRmus(6),STRpole(6)
      Common/STRmasses/STRmass,STRmus,STRpole,NFq

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      a=alphaStr(mpole,nf,nloop)/PI

      zfac=-1975.D0/216.D0*zeta5+1439.D0/432.D0*PI**2*zeta3
     &     -70.D0/27.D0*zeta3+220.D0/27.D0*a4+55.D0/162.D0*LOG(2.D0)**4
     &     +22.D0/81.D0*PI**2*LOG(2.D0)**2+623.D0/162.D0*PI**2*LOG(2.D0)
     &     +695.D0/7776.D0*PI**4-618281.D0/38880.D0*PI**2
     &     -7172965.D0/93312.D0
     &     +nl*(214999.D0/23328.D0+895.D0/648.D0*PI**2
     &     -61.D0/1944.D0*PI**4+11.D0/81.D0*PI**2*LOG(2.D0)
     &     -2.D0/81.D0*PI**2*LOG(2.D0)**2-1.D0/81.D0*LOG(2.D0)**4
     &     -8.D0/27.D0*a4+241.D0/72.D0*zeta3)
     &     +nl**2*(-2353.D0/23328.D0-13.D0/324.D0*PI**2
     &     -7.D0/54.D0*zeta3)

      delfac=0.D0

      Do i=1,nl
         delfac=delfac+STRdelta(STRpole(nl)/mpole)
      End Do

      Pole2SI=mpole*(1.D0
     &     -4.D0/3.D0*a*etal(2)
     &     +a**2*(-2251.D0/288.D0-2.D0*zeta2
     &     -2.D0/3.D0*zeta2*LOG(2.D0)+1.D0/6.D0*zeta3
     &     +nl*(71.D0/144.D0+1.D0/3.D0*zeta2)-4.D0/3.D0*delfac)*etal(3)
     &     +a**3*zfac*etal(4))

      Return
      End

c***********************************************************************

      Double Precision Function SI2Pole(mSI,nf,nl,nloop)

c Function that converts the MS-bar running mass to the pole mass at the
c scale mu with nl lighter quarks in a theory with nf active flavours
c using equation 13 of CKS.

      Implicit NONE

c Inputs
      Double Precision mSI
      Integer nf,nl,nloop

c Local
      Integer i
      Double Precision etal(4)
      Double Precision a,delfac,zfac

c Functions 
      Double Precision alphaStr,STRdelta, MSrmass

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta2
      Parameter(zeta2=1.6449340668482264364724151666460255D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision zeta5
      Parameter(zeta5=1.0369277551433699263313654871823873D0)
      Double Precision a4
      Parameter(a4=0.517479061673899386330758161878231D0)


c Common blocks
      Integer NFq(6)
      Double Precision STRmass(6),STRmus(6),STRpole(6)
      Common/STRmasses/STRmass,STRmus,STRpole,NFq

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      a=alphaStr(mSI,nf,nloop)/PI

      zfac=8481925.D0/93312.D0+652841.D0/38880.D0*PI**2
     &     -695.D0/7776.D0*PI**4-575.D0/162.D0*PI**2*LOG(2.D0)
     &     -22.D0/81.D0*PI**2*LOG(2.D0)**2-55.D0/162.D0*LOG(2.D0)**4
     &     -220.D0/27.D0*a4+58.D0/27.D0*zeta3-1439.D0/432.D0*PI**2*zeta3
     &     +1975.D0/216.D0*zeta5
     &     +nl*(-231847.D0/23328.D0-991.D0/648.D0*PI**2
     &     +61.D0/1944.D0*PI**4-11.D0/81.D0*PI**2*LOG(2.D0)
     &     +2.D0/81.D0*PI**2*LOG(2.D0)**2+1.D0/81.D0*LOG(2.D0)**4
     &     +8.D0/27.D0*a4-241.D0/72.D0*zeta3)
     &     +nl**2*(2353.D0/23328.D0+13.D0/324.D0*PI**2+7.D0/54.D0*zeta3)

      delfac=0.D0

      Do i=1,nl
         delfac=delfac+STRdelta(MSrmass(mSI,i,nf,nloop)/mSI)
      End Do

      SI2Pole=mSI*(1.D0
     &     +4.D0/3.D0*a*etal(2)
     &     +a**2*(307.D0/32.D0+2.D0*zeta2+2.D0/3.D0*zeta2*LOG(2.D0)
     &     -1.D0/6.D0*zeta3
     &     +nl*(-71.D0/144.D0-1.D0/3.D0*zeta2)+4.D0/3.D0*delfac)*etal(3)
     &     +a**3*zfac*etal(4))

      Return
      End

c***********************************************************************
      
      Double Precision Function STRdelta(x)

      Implicit NONE

      Double Precision x

      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)

      STRdelta=PI**2/8.D0*x-0.597D0*x**2+0.230D0*x**3

      Return
      End

c***********************************************************************
      
      Double Precision Function LamDown(Lambda,mSI,nl,nloop)

c Function that determines the value of the strong mass parameter
c Lambda with nl flavours given Lambda with nl+1 flavours and the scale
c invariant mass of the heavy quark.

      Implicit NONE

c Input
      Double Precision Lambda,mSI
      Integer nl,nloop

c Local
      Integer nf,i
      Double Precision etal(4)
      Double Precision b1,b2,b3,b1p,b2p,b3p,lh,tmp

c Functions
      Double Precision beta0,beta1,beta2,beta3
      
c Parameters
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      
      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nf=nl+1
      
      b1=beta1(nf)/beta0(nf)
      b2=beta2(nf)/beta0(nf)
      b3=beta3(nf)/beta0(nf)

      b1p=beta1(nl)/beta0(nl)
      b2p=beta2(nl)/beta0(nl)
      b3p=beta3(nl)/beta0(nl)

      lh=LOG(mSI**2/Lambda**2)

      tmp=(beta0(nl)-beta0(nf))*lh*etal(1)
     &     +(b1p-b1)*LOG(lh)*etal(2)
     &     -b1p*LOG(beta0(nl)/beta0(nf))*etal(2)
     &     +1.D0/(beta0(nf)*lh)*(b1*(b1p-b1)*LOG(lh)+b1p**2-b1**2
     &     -b2p+b2+11.D0/72.D0)*etal(3)
     &     +1.D0/((beta0(nf)*lh)**2)*(-b1**2/2.D0*(b1p-b1)*LOG(lh)**2
     &     +b1*(-b1p*(b1p-b1)+b2p-b2-11.D0/72.D0)*LOG(lh)
     &     +1.D0/2.D0*(-b1p**3-b1**3-b3p+b3)
     &     +b1p*(b1**2+b2p-b2-11.D0/72.D0)
     &     +564731.D0/124416.D0-82043.D0/27648.D0*zeta3
     &     -2633.D0/31104.D0*nl)*etal(4)

      LamDown=Lambda*EXP(tmp/(2.D0*beta0(nl)))

      Return
      End

c***********************************************************************
      
      Double Precision Function LamUp(Lambda,mSI,nf,nloop)

c Function that determines the value of the strong mass parameter
c Lambda with nf flavours given Lambda with nf-1 flavours and the scale
c invariant mass of the heavy quark.

      Implicit NONE

c Input
      Double Precision Lambda,mSI
      Integer nf,nloop

c Local
      Integer nl,i
      Double Precision etal(4)
      Double Precision b1,b2,b3,b1p,b2p,b3p,lh,tmp

c Functions
      Double Precision beta0,beta1,beta2,beta3
      
c Parameters
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do
     
      nl=nf-1
      
      b1=beta1(nf)/beta0(nf)
      b2=beta2(nf)/beta0(nf)
      b3=beta3(nf)/beta0(nf)

      b1p=beta1(nl)/beta0(nl)
      b2p=beta2(nl)/beta0(nl)
      b3p=beta3(nl)/beta0(nl)

      lh=LOG(mSI**2/Lambda**2)

      tmp=(beta0(nf)-beta0(nl))*lh*etal(1)
     &     +(b1-b1p)*LOG(lh)*etal(2)
     &     -b1*LOG(beta0(nf)/beta0(nl))*etal(2)
     &     +1.D0/(beta0(nl)*lh)*(b1p*(b1-b1p)*LOG(lh)+b1**2-b1p**2
     &     -b2+b2p-11.D0/72.D0)*etal(3)
     &     +1.D0/((beta0(nl)*lh)**2)*(-b1p**2/2.D0*(b1-b1p)*LOG(lh)**2
     &     +b1p*(-b1*(b1-b1p)+b2-b2p+11.D0/72.D0)*LOG(lh)
     &     +1.D0/2.D0*(-b1**3-b1p**3-b3+b3p)
     &     +b1*(b1p**2+b2-b2p+11.D0/72.D0)
     &     -564731.D0/124416.D0+82043.D0/27648.D0*zeta3
     &     +2633.D0/31104.D0*nl)*etal(4)

      LamUp=Lambda*EXP(tmp/(2.D0*beta0(nf)))

      Return
      End

c***********************************************************************

      Double Precision Function MassDownSI(mass,mu,mSI,nl,nloop)

c Function that finds the running mass in a theory with nl flavours
c given the mass in a theory with nl+1 flavours, the desired scale
c and the scale invariant mass of the heavy particle.

      Implicit NONE

c Inputs
      Double Precision mass,mu,mSI
      Integer nl,nloop

c Local
      Integer nf,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaSI

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nf=nl+1

      a=alphaStr(mu,nf,nloop)/PI
      lh=LOG(mu**2/mSI**2)

      ZetaSI=1.D0
     &     +a**2*(89.D0/432.D0-5.D0/36.D0*lh+1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(2951.D0/2916.D0-407.D0/864.D0*zeta3+5.D0/4.D0*zeta4
     &     -1.D0/36.D0*B4
     &     +(-1031.D0/2592.D0-5.D0/6.D0*zeta3)*lh+319.D0/432.D0*lh**2
     &     +29.D0/216.D0*lh**3+nl*(1327.D0/11664.D0-2.D0/27.D0*zeta3
     &     -53.D0/432.D0*lh-1.D0/108.D0*lh**3))*etal(4)

      MassDownSI=mass*ZetaSI

      Return
      End

c***********************************************************************

      Double Precision Function MassUpSI(mass,mu,mSI,nf,nloop)

      Implicit NONE

c Function that finds the running mass in a theory with nf flavours
c given the mass in a theory with nf-1 flavours, the desired scale
c and the scale invariant mass of the heavy particle.

c Inputs
      Double Precision mass,mu,mSI
      Integer nf,nloop

c Local
      Integer nl,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaSIInv

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nl=nf-1

      a=alphaStr(mu,nl,nloop)/PI
      lh=LOG(mu**2/mSI**2)

      ZetaSIInv=1.D0
     &     +a**2*(-89.D0/432.D0+5.D0/36.D0*lh-1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(-2951.D0/2916.D0+407.D0/864.D0*zeta3-5.D0/4.D0*zeta4
     &     +1.D0/36.D0*B4
     &     +(133.D0/2592.D0+5.D0/6.D0*zeta3)*lh-155.D0/432.D0*lh**2
     &     -35.D0/216.D0*lh**3+nl*(-1327.D0/11664.D0+2.D0/27.D0*zeta3
     &     +53.D0/432.D0*lh+1.D0/108.D0*lh**3)*etal(4))

      MassUpSI=ZetaSIInv*mass

      Return
      End

c***********************************************************************

      Double Precision Function MassDownPole(mass,mu,mpole,nl,nloop)

c Function that finds the running mass in a theory with nl flavours
c given the mass in a theory with nl+1 flavours, the desired scale
c and the pole mass of the heavy particle.

      Implicit NONE

c Inputs
      Double Precision mass,mu,mpole
      Integer nl,nloop

c Local
      Integer nf,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaPole

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nf=nl+1

      a=alphaStr(mu,nf,nloop)/PI
      lh=LOG(mu**2/mpole**2)

      ZetaPole=1.D0
     &     +a**2*(89.D0/432.D0-5.D0/36.D0*lh+1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(1871.D0/2916.D0-407.D0/864.D0*zeta3+5.D0/4.D0*zeta4
     &     -1.D0/36.D0*B4
     &     +(121.D0/2592.D0-5.D0/6.D0*zeta3)*lh+319.D0/432.D0*lh**2
     &     +29.D0/216.D0*lh**3+nl*(1327.D0/11664.D0-2.D0/27.D0*zeta3
     &     -53.D0/432.D0*lh-1.D0/108.D0*lh**3))*etal(4)

      MassDownPole=mass*ZetaPole

      Return
      End

c***********************************************************************

      Double Precision Function MassUpPole(mass,mu,mpole,nf,nloop)

c Function that finds the running mass in a theory with nf flavours
c given the mass in a theory with nf-1 flavours, the desired scale
c and the pole mass of the heavy particle.

      Implicit NONE

c Inputs
      Double Precision mass,mu,mpole
      Integer nf,nloop

c Local
      Integer nl,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaPoleInv

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nl=nf-1

      a=alphaStr(mu,nl,nloop)/PI
      lh=LOG(mu**2/mpole**2)

      ZetaPoleInv=1.D0
     &     +a**2*(-89.D0/432.D0+5.D0/36.D0*lh-1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(-1871.D0/2916.D0+407.D0/864.D0*zeta3-5.D0/4.D0*zeta4
     &     +1.D0/36.D0*B4
     &     +(-299.D0/2592.D0+5.D0/6.D0*zeta3)*lh-299.D0/432.D0*lh**2
     &     -35.D0/216.D0*lh**3+nl*(-1327.D0/11664.D0+2.D0/27.D0*zeta3
     &     +53.D0/432.D0*lh+1.D0/108.D0*lh**3))*etal(4)

      MassUpPole=ZetaPoleInv*mass

      Return
      End

c***********************************************************************

      Double Precision Function MassDownMS(mass,mu,mMS,nl,nloop)

c Function that finds the running mass in a theory with nl flavours
c given the mass in a theory with nl+1 flavours, the desired scale
c and the running mass at the desired scale of the heavy particle.

      Implicit NONE

c Inputs
      Double Precision mass,mu,mMS
      Integer nl,nloop

c Local
      Integer nf,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaMS

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nf=nl+1

      a=alphaStr(mu,nf,nloop)/PI
      lh=LOG(mu**2/mMS**2)

      ZetaMS=1.D0
     &     +a**2*(89.D0/432.D0-5.D0/36.D0*lh+1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(2951.D0/2916.D0-407.D0/864.D0*zeta3+5.D0/4.D0*zeta4
     &     -1.D0/36.D0*B4
     &     +(-311.D0/2592.D0-5.D0/6.D0*zeta3)*lh+175.D0/432.D0*lh**2
     &     +29.D0/216.D0*lh**3+nl*(1327.D0/11664.D0-2.D0/27.D0*zeta3
     &     -53.D0/432.D0*lh-1.D0/108.D0*lh**3))*etal(4)

      MassDownMS=mass*ZetaMS

      Return
      End

c***********************************************************************

      Double Precision Function MassUpMS(mass,mu,mMS,nf,nloop)

c Function that finds the running mass in a theory with nf flavours
c given the mass in a theory with nf-1 flavours, the desired scale
c and the running mass at the desired scale of the heavy particle.

      Implicit NONE

c Inputs
      Double Precision mass,mu,mMS
      Integer nf,nloop

c Local
      Integer nl,i
      Double Precision etal(4)
      Double Precision a,lh,ZetaMSInv

c Functions
      Double Precision alphaStr

c Parameters
      Double Precision PI
      Parameter (PI=3.1415926535897932384626433832795029D0)
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision B4
      Parameter(B4=-1.762800087073770864061897635009915D0)

      Do i=1,4
         etal(i)=0.D0
      End Do

      Do i=1,nloop
         etal(i)=1.D0
      End Do

      nl=nf-1

      a=alphaStr(mu,nl,nloop)/PI
      lh=LOG(mu**2/mMS**2)

      ZetaMSInv=1.D0
     &     +a**2*(-89.D0/432.D0+5.D0/36.D0*lh-1.D0/12.D0*lh**2)*etal(3)
     &     +a**3*(-2951.D0/2916.D0+407.D0/864.D0*zeta3-5.D0/4.D0*zeta4
     &     +1.D0/36.D0*B4
     &     +(133.D0/2592.D0+5.D0/6.D0*zeta3)*lh-155.D0/432.D0*lh**2
     &     -35.D0/216.D0*lh**3+nl*(-1327.D0/11664.D0+2.D0/27.D0*zeta3
     &     +53.D0/432.D0*lh+1.D0/108.D0*lh**3))*etal(4)

      MassUpMS=ZetaMSInv*mass

      Return
      End

c***********************************************************************
c Formulae for anomalous dimensions and beta function coefficients
c taken from CKS calculated in the MS-bar scheme
c***********************************************************************
      Double Precision Function gamma0(nf)

      Implicit NONE

      Integer nf
      
      gamma0=1.D0

      Return
      End

c***********************************************************************
      Double Precision Function gamma1(nf)

      Implicit NONE

      Integer nf

      gamma1=1.D0/16.D0*(202.D0/3.D0-20.D0/9.D0*nf)

      Return
      End

c***********************************************************************
      Double Precision Function gamma2(nf)

      Implicit NONE

c Inputs
      Integer nf

c Parameters
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)

      gamma2=1.D0/64.D0*(1249.D0+(-2216.D0/27.D0-160.D0/3.D0*zeta3)*nf
     &     -140.D0/81.D0*nf**2)
      
      Return
      End

c***********************************************************************
      Double Precision Function gamma3(nf)

      Implicit NONE
      
c Inputs
      Integer nf
      
c Parameters
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)
      Double Precision zeta4
      Parameter(zeta4=1.0823232337111381915160036965411645D0)
      Double Precision zeta5
      Parameter(zeta5=1.0369277551433699263313654871823873D0)

      gamma3=1.D0/256.D0*(4603055.D0/162.D0+135680.D0/27.D0*zeta3
     &     -8800.D0*zeta5+(-91723.D0/27.D0-34192.D0/9.D0*zeta3
     &     +880.D0*zeta4+18400.D0/9.D0*zeta5)*nf+(5242.D0/243.D0
     &     +800.D0/9.D0*zeta3-160.D0/3.D0*zeta4)*nf**2
     &     +(-332.D0/243.D0+64.D0/27.D0*zeta3)*nf**3)

      Return
      End

c***********************************************************************

      Double Precision Function beta0(nf)

      Implicit NONE

c Inputs      
      Integer nf

      beta0=1.D0/4.D0*(11.D0-2.D0/3.D0*nf)

      Return
      End

c***********************************************************************
      Double Precision Function beta1(nf)

      Implicit NONE

c Inputs
      Integer nf

      beta1=1.D0/16.D0*(102.D0-38.D0/3.D0*nf)

      Return
      End

c***********************************************************************
      Double Precision Function beta2(nf)

      Implicit NONE

c Inputs
      Integer nf

      beta2=1.D0/64.D0*(2857.D0/2.D0-5033.D0/18.D0*nf
     &     +325.D0/54.D0*nf**2)

      Return
      End

c***********************************************************************
      Double Precision Function beta3(nf)

      Implicit NONE

c Inputs
      Integer nf

c Parameters
      Double Precision zeta3
      Parameter(zeta3=1.2020569031595942853997381637067650D0)

      beta3=1.D0/256.D0*(149753.D0/6.D0+3564*zeta3-(1078361.D0/162.D0
     &     +6508.D0/27.D0*zeta3)*nf+(50065.D0/162.D0
     &     +6472.D0/81.D0*zeta3)*nf**2+1093.D0/729.D0*nf**3)

      Return
      End
