/*
	lntree.c
		sets up a build tree with links to the
		files in the specified source directories
		last modified 6 Jun 06 th

	Syntax: lntree <targetdir> <srcdir1> <srcdir2> ...
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>

#define BLOCK 10240


static char *mkdirhier(char *dir, char *rel)
{
  struct stat st;
  char *s = dir, *e = dir + strlen(dir) - 1;

  if( *e != '/' ) *++e = '/';

  do {
    s = strchr(s, '/');
    *s = 0;
    if( stat(dir, &st) ) mkdir(dir, 0777);
    else if( !S_ISDIR(st.st_mode) ) {
      fprintf(stderr, "Cannot create %s\n", dir);
      exit(1);
    }
    *rel++ = '.';
    *rel++ = '.';
    *rel++ = '/';
    *s++ = '/';
  } while( s < e );

  *e = 0;
  return rel;
}


static char *ls(const char *dir)
{
  int size = 0, pos = 0;
  char *files = NULL;
  struct dirent *entry;
  DIR *cwd = opendir(dir);

  if( cwd == NULL ) return strdup("");

  while( (entry = readdir(cwd)) ) {
    char *file = entry->d_name;

    if( *file != '.' ) {
      const int len = strlen(file);

      /* omit the .o files */
      if( len > 2 && file[len - 2] == '.' && file[len - 1] == 'o' )
        continue;

      if( pos + len + 1 > size ) {
        size += BLOCK;
        files = realloc(files, size);
        if( files == NULL ) {
          fprintf(stderr, "Out of memory\n");
          exit(1);
        }
      }

      memcpy(files + pos, file, len);
      pos += len;
      files[pos++] = '|';
    }
  }

  closedir(cwd);

  if( files == NULL ) return strdup("");

  files[pos] = 0;
  return files;
}


int main(int argc, char **argv)
{
  int arg;
  char relpath[PATH_MAX], *rel;
  char *lstarget, *source[128];

  if( argc < 3 ) {
    fprintf(stderr, "Usage:\t%s targetdir srcdir1 srcdir2...\n"
      "\tcreates targetdir and in it creates symlinks to all\n"
      "\tfiles in srcdir1, srcdir2, ...\n", argv[0]);
    exit(1);
  }

  rel = mkdirhier(argv[1], relpath);
  lstarget = ls(argv[1]);

  memcpy(source, argv + 2, (argc -= 2)*sizeof(char *));

  for( arg = 0; arg < argc; ++arg ) {
    char *lssource = ls(source[arg]);
    char *cur = lssource, *next;

    while( (next = strchr(cur, '|')) ) {
      *next = 0;
      if( strstr(lstarget, cur) == NULL ) {
        const int n = sprintf(rel, "%s/%s", source[arg], cur);
        struct stat st;
        if( stat(rel, &st) == 0 ) {
          if( S_ISDIR(st.st_mode) ) {
            if( argc >= sizeof(source)/sizeof(source[0]) ) {
              fprintf(stderr, "Too many source directories\n");
              exit(1);
            }
            memcpy(source[argc++] = malloc(n + 1), rel, n + 1);
          }
          else if( S_ISREG(st.st_mode) ) {
            char dlink[PATH_MAX];
            sprintf(dlink, "%s/%s", argv[1], cur);
            symlink(relpath, dlink);
          }
        }
      }
      cur = next + 1;
    }

    free(lssource);
  }

  return 0;
}

