// Interfacing routine SOFTSUSY-MCMC
// Author: Roberto Ruiz

#include <iostream>
#include <iomanip>
#include <math.h>
#include "mycomplex.h"
#include "def.h"
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"
#include "interf.h"

/// global variable declaration
// no quark mixing, and no verbose output
int MIXING = 0, PRINTOUT = 0;
// fractional accuracy required
double TOLERANCE = 1.0e-3;
// decay constant of muon
double GMU = 1.16637e-5; 
// M_{SUSY}/root(mstop1 mstop2). The default value is 1
double QEWSB = 1.; 
/// Do we include 2-loop RGEs of *all* scalar masses and A-terms, or only the
/// scalar mass Higgs parameters? (Other quantities all 2-loop anyway): the
/// default in SOFTSUSY 2.x is to include all 2-loop terms
bool INCLUDE_2_LOOP_SCALAR_CORRECTIONS = true;
double scaledep;
// number of loops used to calculate Higgs mass and tadpoles. They should be
//identical for a consistent calculation
int numHiggsMassLoops = 2, numRewsbLoops = 2;
/// end of global variable declaration


extern "C"

void softsusy_(flags & flag, cmssm & input_cmssm, nuisance & input_nuis, 
            susy_spec & spec, softsusy & outsoft, leshouches_out & leshouches, 
            error & Out_err) 
{


 // Sets format of output: 4 decimal places
  outputCharacteristics(6);
 

  double mGutGuess = 2.0e16;


  QedQcd oneset;      // See "lowe.h" for default definitions of SM parameters


  outsoft.sm.mUp = oneset.displayMass(mUp);
  outsoft.sm.mCharm = oneset.displayMass(mCharm);
  outsoft.sm.mDown = oneset.displayMass(mDown);
  outsoft.sm.mStrange = oneset.displayMass(mStrange);
  outsoft.sm.mElectron = oneset.displayMass(mElectron);
  outsoft.sm.mMuon = oneset.displayMass(mMuon);
  outsoft.sm.mTau = oneset.displayMass(mTau);


  oneset.setAlpha(ALPHA, input_nuis.alphamz);
  oneset.setAlpha(ALPHAS, input_nuis.alphasmz);
  oneset.setPoleMt(input_nuis.mtop);
  oneset.setMass(mBottom,input_nuis.mbottom);
  oneset.setMass(mTop,getRunMtFromMz(oneset.displayPoleMt(),
                                     oneset.displayAlpha(ALPHAS)));


  oneset.toMz();      // Runs SM fermion masses to MZ


// Preparation for calculation: set up object and input parameters

  MssmSoftsusy r; 
  DoubleVector pars(3); 
  pars(1) = input_cmssm.m0; pars(2) = input_cmssm.m12; 
  pars(3) = input_cmssm.a0;
  bool uni = true; // Unify gauge couplings

  scaledep = input_cmssm.scaledep;


// Calculate the spectrum
  double mx;
  mx = r.lowOrg(sugraBcs, mGutGuess, pars, input_cmssm.sgnMu, 
                input_cmssm.tanb, oneset, uni);


       
// Output the mssmsoftsusy object for debugging
// cout << r << endl; 

  
  
// Softterms 
  outsoft.soft.mu = r.displaySusyMu();
  outsoft.soft.m1 = r.displayGaugino(1);
  outsoft.soft.m2 = r.displayGaugino(2);


  int j=0;
  for (int i=0; i<=2; i++){
      j++;
      outsoft.soft.asofte[i] = r.displaySoftA(EA, j, j); 
      outsoft.soft.asoftu[i] = r.displaySoftA(UA, j, j);
      outsoft.soft.asoftd[i] = r.displaySoftA(DA, j, j);

      outsoft.soft.mass2l[i] = r.displaySoftMassSquared(mLl, j, j);
      outsoft.soft.mass2e[i] = r.displaySoftMassSquared(mEr, j, j);
      outsoft.soft.mass2q[i] = r.displaySoftMassSquared(mQl, j, j);
      outsoft.soft.mass2u[i] = r.displaySoftMassSquared(mUr, j, j);
      outsoft.soft.mass2d[i] = r.displaySoftMassSquared(mDr, j, j);       
     }


 


// Storing spectrum passed to ds

// Sfermion sector

  for(int i = 0; i <= 1; i++) {
      for( int g = 0; g <= 2; g++) {
         outsoft.sfer.msu[i][g] = r.displayPhys().mu(i+1,g+1);
         outsoft.sfer.msd[i][g] = r.displayPhys().md(i+1,g+1);
         outsoft.sfer.mse[i][g] = r.displayPhys().me(i+1,g+1);
         outsoft.sfer.msnu[g] = r.displayPhys().msnu(g+1);
       }
   }



  outsoft.sfer.thetat =  r.displayPhys().thetat;
  outsoft.sfer.thetab = r.displayPhys().thetab;
  outsoft.sfer.thetatau = r.displayPhys().thetatau;


// Neutralino sector

  for(int i = 0; i <= 3; i++) {
      outsoft.neut.mneut[i] =  r.displayPhys().mneut(i+1); 
     }

  for(int i=0; i <=3; i++) {
      for(int j=0; j <=3; j++) {    
	   outsoft.neut.mixchi[i][j]=r.displayPhys().mixNeut(i+1,j+1); 
		 }
     }

    

// Charg sector

  for(int i = 0; i <= 1; i++) {
	 outsoft.charg.mcha[i] =  r.displayPhys().mch(i+1); 
     }


  outsoft.charg.thetaL = r.displayPhys().thetaL;
  outsoft.charg.thetaR = r.displayPhys().thetaR;


// Higgs sector

  outsoft.higgs.ma = r.displayPhys().mhiggs.display(2);
  outsoft.higgs.mh = r.displayPhys().mhiggs.display(1);
  outsoft.higgs.mH = r.displayPhys().mhiggs.display(3);
  outsoft.higgs.mcH = r.displayPhys().mhiggs.display(4);
  outsoft.higgs.halpha= r.displayPhys().thetaH;

  outsoft.mgluino = r.displayPhys().mGluino;


//    cout << r.displayProblem().testSeriousProblem() << endl;
//    cout << r.displayProblem().test() << endl;

// Init. error flags 

  int err = 0;

  Out_err.muSqWrongSign = 0;

  Out_err.b = 0;

  Out_err.noRhoConvergence = 0;    

  Out_err.tachyon  = 0;

  Out_err.nonperturbative = 0;

  Out_err.noMuConvergence  = 0;

  Out_err.higgsUfb = 0;

  Out_err.irqfp = 0;

  Out_err.badConvergence = 0;


// Error flags output

 
  if(r.displayProblem().test()) err = 1;

  if(r.displayProblem().muSqWrongSign) Out_err.muSqWrongSign = 1;

  if(r.displayProblem().b) Out_err.b = 1;

  if(r.displayProblem().noRhoConvergence) Out_err.noRhoConvergence = 1;

  if(r.displayProblem().tachyon)  Out_err.tachyon  = 1;

  if(r.displayProblem().nonperturbative) Out_err.nonperturbative = 1;

  if(r.displayProblem().noMuConvergence) Out_err.noMuConvergence  = 1;

  if(r.displayProblem().higgsUfb) Out_err.higgsUfb = 1;

  if(r.displayProblem().irqfp) Out_err.irqfp = 1;
   
  if(r.displayProblem().badConvergence || r.displayProblem().noConvergence) 
   Out_err.badConvergence = 1;

  
    
// Physical spectrum for output 

// Higgs sector

  spec.mlh = r.displayPhys().mhiggs.display(1);
  spec.mhh = r.displayPhys().mhiggs.display(3);
  spec.mA = r.displayPhys().mhiggs.display(2);
  spec.mhc = r.displayPhys().mhiggs.display(4);   

 
  double beta = atan(input_cmssm.tanb);
  double alphah = r.displayPhys().thetaH;

  spec.SinBmASq = sqr(sin(beta-alphah));


// Chargino sector

  spec.mch1 = minimum(fabs(r.displayPhys().mch(1)),
                        fabs(r.displayPhys().mch(2)));
  spec.mch2 = maximum(fabs(r.displayPhys().mch(1)),
                        fabs(r.displayPhys().mch(2)));    

// Neutralinos

  double mlsp; 

  spec.mneut1 = fabs(r.displayPhys().mneut(1)); 

  mlsp = spec.mneut1;

  spec.mneut2 = fabs(r.displayPhys().mneut(2));

  spec.mneut3 = fabs(r.displayPhys().mneut(3));       

  spec.mneut4 = fabs(r.displayPhys().mneut(4));

// Sleptons

  spec.mseR = minimum(r.displayPhys().me(1,1),r.displayPhys().me(2,1));
  spec.mseL = maximum(r.displayPhys().me(1,1),r.displayPhys().me(2,1)); 

  spec.msmuR = minimum(r.displayPhys().me(1,2),r.displayPhys().me(2,2));
  spec.msmuL = maximum(r.displayPhys().me(1,2),r.displayPhys().me(2,2)); 

  spec.mstau1 = minimum(r.displayPhys().me(1,3),r.displayPhys().me(2,3));
  spec.mstau2 = maximum(r.displayPhys().me(1,3),r.displayPhys().me(2,3));   

// Sneutrinos

  spec.msne  = r.displayPhys().msnu(1);

  spec.msnmu  = r.displayPhys().msnu(2);

  spec.msntau  = r.displayPhys().msnu(3);

// Squarks

// Up

  spec.msuR = minimum(r.displayPhys().mu(1,1),r.displayPhys().mu(2,1));
  spec.msuL = maximum(r.displayPhys().mu(1,1),r.displayPhys().mu(2,1));

  spec.mscR = minimum(r.displayPhys().mu(1,2),r.displayPhys().mu(2,2));
  spec.mscL = maximum(r.displayPhys().mu(1,2),r.displayPhys().mu(2,2));

  spec.mst1 = minimum(r.displayPhys().mu(1,3),r.displayPhys().mu(2,3));
  spec.mst2 = maximum(r.displayPhys().mu(1,3),r.displayPhys().mu(2,3));

// Down

  spec.msdR = minimum(r.displayPhys().md(1,1),r.displayPhys().md(2,1));
  spec.msdL = maximum(r.displayPhys().md(1,1),r.displayPhys().md(2,1));

  spec.mssR = minimum(r.displayPhys().md(1,2),r.displayPhys().md(2,2));
  spec.mssL = maximum(r.displayPhys().md(1,2),r.displayPhys().md(2,2));

  spec.msb1 = minimum(r.displayPhys().md(1,3),r.displayPhys().md(2,3));
  spec.msb2 = maximum(r.displayPhys().md(1,3),r.displayPhys().md(2,3));


// Gluino

  spec.mgluino = r.displayPhys().mGluino;


// Running masses including SUSY threshold corrections

  if (err == 0) {
      r.calcDrBarPars();
      outsoft.run.mtaumz = r.calcRunningMtau();
      outsoft.run.mbmz = r.calcRunningMb();
      outsoft.run.mtmt = r.calcRunningMt();
    }
  else {
      outsoft.run.mtaumz = 0.0;
      outsoft.run.mbmz = 0.0;
      outsoft.run.mtmt = 0.0;
    };


// Top and bottom quark pole masses 


  outsoft.run.mbpole=oneset.displayPoleMb();
  outsoft.run.mtpole=oneset.displayPoleMt();

  
// Yukawas at the neutralino mass scale for neut annihilation...

  if (2.*mlsp > r.displayMz()) r.runto(2.*mlsp);

  if (err == 0) {
    outsoft.run.ylsp_tau=r.displayYukawaElement(YE, 3, 3);
    outsoft.run.ylsp_b=r.displayYukawaElement(YD, 3, 3);
    outsoft.run.ylsp_top=r.displayYukawaElement(YU, 3, 3);}
  else {
    outsoft.run.ylsp_tau= 0.0;
    outsoft.run.ylsp_b= 0.0;
    outsoft.run.ylsp_top = 0.0;
    };




//LesHouches output for FH

if(abs(flag.fh_predic)) {

  QedQcd d(r.displayDataSet());
//  Standard Model inputs 
  leshouches.SMinputs.GFermi = GMU;
// G_Fermi 
  leshouches.SMinputs.alpha_s= d.displayAlpha(ALPHAS);
// alpha_s(MZ)MSbar 
  leshouches.SMinputs.MZ =  MZ;
// MZ(pole) 
  leshouches.SMinputs.mb =  d.displayPoleMb(); //input_nuis.mbottom; 
// mb(mb) 
  leshouches.SMinputs.mtop =  d.displayPoleMt();
// Mtop(pole) 
  leshouches.SMinputs.mtau = MTAU;
// Mtau(pole) 


  sPhysical s(r.displayPhys());

// Mass spectrum 

  leshouches.mass.MW = r.displayMw();
// MW";
  leshouches.mass.mh0 = s.mhiggs(1);
// h0";
  leshouches.mass.mH0 = s.mhiggs(3);
// H0 
  leshouches.mass.mA0 =  s.mhiggs(2); 
// A0 
  leshouches.mass.mHp =  s.mhiggs(4);
// H+ 
  leshouches.mass.msdl= s.md(1, 1);
// ~d_L 
  leshouches.mass.msul = s.mu(1, 1);
// ~u_L 
  leshouches.mass.mssl =s.md(1, 2);
// ~s_L 
  leshouches.mass.mscl= s.mu(1, 2);
// ~c_L 
  leshouches.mass.msb1 =  minimum(s.md(1, 3), s.md(2, 3));
// ~b_1  
  leshouches.mass.mst1 = minimum(s.mu(1, 3), s.mu(2, 3));
// ~t_1 

  leshouches.mass.msdr = s.md(2, 1);
// ~d_R 
  leshouches.mass.msur = s.mu(2, 1); 
// ~u_R 
  leshouches.mass.mssr =  s.md(2, 2); 
// ~s_R 
  leshouches.mass.mscr = s.mu(2, 2);
// ~c_R 
  leshouches.mass.msb2 =  maximum(s.md(1, 3), s.md(2, 3));
// ~b_2 
  leshouches.mass.mst2 =  maximum(s.mu(1, 3), s.mu(2, 3));
// ~t_2 


  leshouches.mass.msel =  s.me(1, 1);
//  ~e_L 
  leshouches.mass.msnue =  s.msnu(1);
//  ~nue_L 
  leshouches.mass.msmul = s.me(1, 2);
// ~mu_L 
  leshouches.mass.msnumu = s.msnu(2);
// ~numu_L 
  leshouches.mass.mstau1 = minimum(s.me(1, 3), s.me(2, 3));
// ~stau_1 
  leshouches.mass.msnutau =  s.msnu(3);
// ~nu_tau_L 

  leshouches.mass.mser =  s.me(2, 1);
//  "# ~e_R 
  leshouches.mass.msmur =  s.me(2, 2);
// ~mu_R 
  leshouches.mass.mstau2 =  maximum(s.me(1, 3), s.me(2, 3));
// stau_2 

  leshouches.mass.mgluino = s.mGluino;
// ~g 
  leshouches.mass.mneut1 = s.mneut(1);
// neutralino(1) 
  leshouches.mass.mneut2 = s.mneut(2);
// neutralino(2) 
  leshouches.mass.mneut3 = s.mneut(3);
// neutralino(3) 
  leshouches.mass.mneut4 = s.mneut(4);
// neutralino(4)  
  leshouches.mass.mch1 = s.mch(1);
// chargino(1) 
  leshouches.mass.mch2 = s.mch(2);
// chargino(2) 



// Effective Higgs mixing parameter 
  leshouches.mass.alpha =  s.thetaH; 
  

  double ms = r.displayMsusy();

  DoubleMatrix m(2, 2);

//stop mixing matrix 
  if (s.mu(1, 3) < s.mu(2, 3)) m = rot2d(s.thetat);
  else m = rot2dTwist(s.thetat);
  int i, j; 
  for (i=0; i<=1; i++)
    for (j=0; j<=1; j++) {
      leshouches.mix.StopMix[i][j] = m(i+1, j+1);
    }


//sbottom mixing matrix 
  if (s.md(1, 3) < s.md(2, 3)) m = rot2d(s.thetab);
  else m = rot2dTwist(s.thetab);
  for (i=0; i<=1; i++)
    for (j=0; j<=1; j++) {
      leshouches.mix.SbotMix[i][j] = m(i+1, j+1);
    }


//stau mixing matrix 
  if (s.me(1, 3) < s.me(2, 3)) m = rot2d(s.thetatau);
  else m = rot2dTwist(s.thetatau);
  for (i=0; i<=1; i++)
    for (j=0; j<=1; j++) {
      leshouches.mix.StauMix[i][j] = m(i+1, j+1);
   }



//neutralino mixing matrix 
  for (i=0; i<=3; i++)
    for (j=0; j<=3; j++) {
      leshouches.mix.NMix[i][j] = s.mixNeut(j+1, i+1);
    }
  

  
  DoubleMatrix u(rot2d(s.thetaL)), v(rot2d(s.thetaR)); 


// chargino U mixing matrix  
  for (i=0; i<=1; i++)
    for (j=0; j<=1; j++) {
      leshouches.mix.UMix[i][j] = u(i+1, j+1);
    }

// chargino V mixing matrix  
  for (i=0; i<=1; i++)
    for (j=0; j<=1; j++) {
      leshouches.mix.VMix[i][j] = v(i+1, j+1);
    }


    r.runto(ms);
    double gp = r.displayGaugeCoupling(1) * sqrt(0.6);

// Renormalization cale

    leshouches.scale.Q =  ms; 

//  Block gauge Q
    leshouches.coup.gp = gp;
//  g'(Q)MSSM DRbar
    leshouches.coup.g2 = r.displayGaugeCoupling(2);
//  g(Q)MSSM DRbar
    leshouches.coup.g3 = r.displayGaugeCoupling(3);  
//  g3(Q)MSSM DRbar
    
    leshouches.coup.ytq = r.displayYukawaElement(YU, 3, 3);
//  Yt(Q)MSSM DRbar
    leshouches.coup.ybq = r.displayYukawaElement(YD, 3, 3);
//  Yd(Q)MSSM DRbar
    leshouches.coup.ytauq = r.displayYukawaElement(YE, 3, 3);
//  Ytau(Q)MSSM DRbar


//  SUSY parameters:
    leshouches.susy.mu = r.displaySusyMu();
//  mu(Q)MSSM DRbar 
    leshouches.susy.tanb = r.displayTanb();
//  tan beta(Q)MSSM DRbar 
    leshouches.susy.hvev = r.displayHvev();
//  higgs vev(Q)MSSM DRbar 
 

//MSSM DRbar SUSY breaking parameters at Q: 

    leshouches.soft.m3sq = r.displayM3Squared()/ 
     (sin(atan(r.displayTanb())) * cos(atan(r.displayTanb())));
//  mA^2(Q)MSSM DRbar 

    leshouches.soft.M1 = r.displayGaugino(1);
    leshouches.soft.M2 = r.displayGaugino(2);
    leshouches.soft.M3 = r.displayGaugino(3);
//  Gaugino masses: M_i(Q)
 

    leshouches.soft.mH12= r.displayMh1Squared(); 
//  mH1^2(Q)
    leshouches.soft.mH22= r.displayMh2Squared();
//  mH2^2(Q)
    leshouches.soft.meL = ccbSqrt(r.displaySoftMassSquared(mLl, 1, 1));
//  meL(Q)
    leshouches.soft.mmuL = ccbSqrt(r.displaySoftMassSquared(mLl, 2, 2));
//  mmuL(Q)
    leshouches.soft.mtauL = ccbSqrt(r.displaySoftMassSquared(mLl, 3, 3));
//  mtauL(Q)
    leshouches.soft.meR = ccbSqrt(r.displaySoftMassSquared(mEr, 1, 1));
//  meR(Q)
    leshouches.soft.mmuR = ccbSqrt(r.displaySoftMassSquared(mEr, 2, 2));
//  mmuR(Q)
    leshouches.soft.mtauR = ccbSqrt(r.displaySoftMassSquared(mEr, 3, 3));
//  mtauR(Q)
    leshouches.soft.mqL1 = ccbSqrt(r.displaySoftMassSquared(mQl, 1, 1));
//  mqL1(Q)
    leshouches.soft.mqL2 = ccbSqrt(r.displaySoftMassSquared(mQl, 2, 2));
//  mqL2(Q)
    leshouches.soft.mqL3 = ccbSqrt(r.displaySoftMassSquared(mQl, 3, 3));
//  mqL3(Q)
    leshouches.soft.muR = ccbSqrt(r.displaySoftMassSquared(mUr, 1, 1));
//  muR(Q)
    leshouches.soft.mcR = ccbSqrt(r.displaySoftMassSquared(mUr, 2, 2));
//  mcR(Q)
    leshouches.soft.mtR = ccbSqrt(r.displaySoftMassSquared(mUr, 3, 3));
//  mtR(Q)
    leshouches.soft.mdR = ccbSqrt(r.displaySoftMassSquared(mDr, 1, 1));
//  mdR(Q)
    leshouches.soft.msR = ccbSqrt(r.displaySoftMassSquared(mDr, 2, 2));
//  msR(Q)
    leshouches.soft.mbR = ccbSqrt(r.displaySoftMassSquared(mDr, 3, 3));
//  mbR(Q)



//  Au Q:

    leshouches.soft.aup = r.displaySoftA(UA, 1, 1);
//  Au(Q)MSSM DRbar
    leshouches.soft.ach = r.displaySoftA(UA, 2, 2);
//  Ac(Q)MSSM DRbar
    leshouches.soft.at = r.displaySoftA(UA, 3, 3);
//  At(Q)MSSM DRbar


//  Ad Q:
 
    leshouches.soft.ado = r.displaySoftA(DA, 1, 1);
//  Ad(Q)MSSM DRbar
    leshouches.soft.ast = r.displaySoftA(DA, 2, 2);
//  As(Q)MSSM DRbar
    leshouches.soft.ab = r.displaySoftA(DA, 3, 3);
//  Ab(Q)MSSM DRbar

    

//  Ae Q:

    leshouches.soft.ael = r.displaySoftA(EA, 1, 1);
//  Ae(Q)MSSM DRbar
    leshouches.soft.amu = r.displaySoftA(EA, 2, 2);
//  Amu(Q)MSSM DRbar
    leshouches.soft.atau = r.displaySoftA(EA, 3, 3);
//  Atau(Q)MSSM DRbar

    
    r.runto(input_nuis.mtop);
    //tanb is DRbar at mtop as recomended value
    leshouches.susy.tanb = r.displayTanb();

}

}
