! Module containing handling of MCMC samples - largely from the GetDist routine of CosmoMC  
! by Anthony Lewis & Sarah Bridle (cosmologist.info/cosmomc)
!Output files are largely as in the original GetDist (refer to cosmologist.info/cosmomc for 
! details)
! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version May 2007 !

module MCSamples

  use settings
  use Random
  implicit none

  integer, parameter :: gp = KIND(1.d0)
  
  real(gp), dimension(:,:), target, allocatable :: coldata  !(col_index, row_index)
  integer, dimension(:), allocatable :: thin_ix  
  integer thin_rows
  
  integer, parameter :: max_rows = 2000000
  integer, parameter :: max_cols = 200
  integer, parameter :: max_chains = 300
  integer, parameter :: max_split_tests = 5 
  integer, parameter :: max_failures = 10
  
  integer chain_indices(max_chains), num_chains_used
  
  integer nrows, ncols,nmapped, norig
  real numsamp
  integer ND_cont1, ND_cont2
  integer covmat_dimension
  integer colix(max_cols), num_vars  !Parameters with non-blank lables
  real mean(max_cols), sddev(max_cols)
  real, dimension(:,:), allocatable :: corrmatrix
  character(LEN=120) rootname
  character(LEN=120) lables(max_cols)
  logical has_limits(max_cols), has_limits_top(max_cols),has_limits_bot(max_cols)
  integer :: tails(max_cols) = 2
  integer num_contours 
  real contours(2), contour_levels(2)
  real mean_mult, max_mult
  integer :: indep_thin = 0 
  integer chain_numbers(max_chains)
  logical isused(max_cols)
  logical force_twotail 
  logical smoothing, plot_meanlikes
  logical :: plot_NDcontours = .false.
  logical shade_meanlikes, make_single_samples
  integer single_thin,cust2Dplots(max_cols**2)
  integer ix_min(max_cols),ix_max(max_cols)
  real center(max_cols),width(max_cols)   
  real, dimension(:,:), allocatable :: TheBins, bins2D, bin2Dlikes, bin2Dmax
  real meanlike, maxlike, maxmult
  logical BW,do_shading
  character(LEN=80) ComparePlots(20)
  integer Num_ComparePlots
  logical ::   prob_lable = .false.      
  
  
  character(LEN=80) dummystring
  integer :: skip_bin
  
  !output files directory
  character(LEN=100) :: outn = 'output_files/'
  

contains

  subroutine AdjustPriors
    !Can adjust the multiplicity of each sample in coldata(1, rownum) for new priors
    !Be careful as this code is parameterisation dependent
    integer  :: i, cutaway = 0
    real chisq

    stop 'You need to write the AdjustPriors subroutine in GetDist.F90 first!'

    !Just an example:
    !here I cut away ridicoulosuly large values of theta_b
    !write (*,*) 'Adjusting priors'
    !do i=0, nrows-1
       !if (abs(invars(2+90)) > 10.0) then
       ! invars(1) = 0d0
       ! invars(2) = LogZero
       ! cutaway = cutaway+1
       ! write(*,*) 'cutpoints = ', cutaway
       !end if
    !end do
    !write(*,*) 'AdjustPrior: cutpoints = ', cutaway, ', percent of total', 1.0*cutaway/(1.0*nrows)  

  end subroutine AdjustPriors

  subroutine MapParameters(invars)
    real invars(1:ncols),tmp, sigmanuc, brmumu, mh,chisq
    integer :: pos_sigma_nucl, pos_brmumu, pos_mh 
    integer :: start_col, end_col, write_col,l
    real, save :: multsamp = 0.0
    integer :: xb, xt
    logical, save :: adjustlab = .true.
    real*8 :: sigma_CMSSM, sigma_SM, sigmaBR_CMSSM, sigmaBR_SM

    !stop 'Need to write MapParameters routine first'  
    !Here for example I convert from a linear to a Log Scale
    !the position in the chain does depend on what you save
    !look in the .info file
    !I assume here that all of the compute= flags are set to T
    pos_sigma_nucl = 12
    pos_brmumu = 17

    if (adjustlab) then
       lables(2+pos_sigma_nucl) = 'log('//trim(lables(2+pos_sigma_nucl))//')'
       lables(2+pos_brmumu) = 'log('//trim(lables(2+pos_brmumu))//')'
       adjustlab = .false.
    end if

    sigmanuc=invars(2+pos_sigma_nucl)
    brmumu=invars(2+pos_brmumu)

    !overwrites with the log10 of the quantities
    invars(2+pos_sigma_nucl) = LOG10(sigmanuc)
    invars(2+pos_brmumu) = LOG10(brmumu)

    adjustlab = .false.
  end subroutine MapParameters

  subroutine CoolChain(cool)
    real, intent(in) :: cool
    integer i
    real maxL, newL
    
    write (*,*) 'Cooling chains by ', cool
    MaxL = minval(coldata(2,0:nrows-1))
    do i=0, nrows-1
     newL = coldata(2,i)*cool
     coldata(1,i) = coldata(1,i)*exp(-(newL - coldata(2,i)) - MaxL*(1-cool) )
     coldata(2,i) = newL
   end do

  end subroutine CoolChain

  subroutine AddParameters(invars)
    real invars(1:ncols)
    integer :: A1, A2, B, C, D
    integer :: Q, T, W, xb, xt
    integer :: write_col
    
    !stop 'You need to write AddParameters routine first'  

    !below is an example as to how you might want to do it
    !the order does depend on what you have saved on file!
    A1 = 2+58 !OK
    A2 = 2+54 !OK
    B = 2+66  !OK
    C = 2+80  !OK
    D = 2+62  !OK
    
    Q = 2+83 !OK
    T = 2+87 !OK
    W = 2+89 !OK
    
    !now compute derived quantities to be added at the end:
    write_col = norig+1
    invars(write_col)   = invars(A1)*invars(Q)
    invars(write_col+1) = invars(A1)*invars(T)
 
  end subroutine AddParameters


  subroutine DeleteZeros
    integer i,ii

    ii=0
    do i=0, nrows-1
     if (coldata(1,i)/=0) then
       coldata(:,ii) = coldata(:,i)
       ii=ii+1
     end if
     end do
     if (ii==0) stop 'Prior has removed all models!'
     if (ii /= nrows) write (*,*) 'Prior removed ',nrows-ii,'models'
     nrows = ii
          
  end subroutine DeleteZeros


  subroutine SortColData(bycol)
    !Sort coldata in order of likelihood
    Type(double_pointer), dimension(:), allocatable :: rowptrs
     integer, intent(in) :: bycol
     real(gp), dimension(:,:), pointer :: tmp 
     integer i

     allocate(tmp(ncols,0:nrows-1))
  
     allocate(rowptrs(0:nrows-1))
     do i = 0, nrows -1
      rowptrs(i)%p => coldata(1:,i)
     end do
     call QuickSortArr(rowptrs(0), 1, nrows, bycol)
     !Make new table using sorted pointers   
     do i = 0, nrows-1
        tmp(1:ncols,i) = rowptrs(i)%p(1:ncols)
     end do
     
     deallocate(rowptrs)
     do i=0, nrows-1
        !Write this out explicitly to avoid stack overflow problems
        coldata(1:ncols,i) = tmp(1:ncols,i)
     end do
     deallocate(tmp)

   end  subroutine SortColData


   subroutine MakeSingleSamples(single_thin)
     
     !Make file of weight-1 samples by choosing samples with probability given by their weight
     use Random
     integer i, single_thin
     character(LEN=20) :: gfmt
     real maxmult
     Feedback = 0          
     call initRandom()
     gfmt = trim(numcat('(',ncols))//'E15.5)'
     open(unit=50,file=trim(outn)//trim(rootname)//'_single.txt',form='formatted',status='replace')
     maxmult = maxval(coldata(1,0:nrows-1))
     do i= 0, nrows -1
        if (ranmar() <= coldata(1,i)/maxmult/single_thin) write (50,gfmt) 1.0, coldata(2:ncols,i)          
     end do
     close(50)
     
   end subroutine MakeSingleSamples

   subroutine WriteThinData(fname,cool)
     character(LEN=*), intent(in) :: fname
     real,intent(in) :: cool
     character(LEN=20) :: gfmt
     integer i
     real MaxL, NewL      

     if (cool /= 1) write (*,*) 'Cooled thinned output with temp: ', cool
     MaxL = minval(coldata(2,0:nrows-1))
     
     gfmt = trim(numcat('(',ncols))//'E15.5)'
     open(unit=50,file=fname,form='formatted',status='replace')
    
     do i=0, thin_rows -1   
        if (cool/=1) then
           newL = coldata(2,thin_ix(i))*cool
           write (50,gfmt) exp(-(newL - coldata(2,thin_ix(i))) - MaxL*(1-cool) ), newL, &
                coldata(3:ncols,thin_ix(i))
        else 
           write (50,gfmt) 1., coldata(2:ncols,thin_ix(i)) 
        end if
     end do
     write (*,*) 'Wrote ',thin_rows, 'thinned samples'
     close(50)
  
   end subroutine WriteThinData


   subroutine ThinData(fac, ix1,ix2)
     !Make thinned samples
     
     integer, intent(in) :: fac
     integer, intent(in), optional :: ix1,ix2
     integer i, tot, nout, nend, mult
          
     if (allocated(thin_ix)) deallocate(thin_ix)
     allocate(thin_ix(0:nint(numsamp/fac)+1))
     
     tot= 0
     nout=0
     i = 0
     if (present(ix1)) i=ix1
     nend = nrows
     if (present(ix2)) nend = ix2+1
     
     mult = coldata(1,i)
     do while (i< nend)
        if (abs(nint(coldata(1,i)) - coldata(1,i)) > 1e-4) &
             stop 'non-integer weights in ThinData'
        
        if (mult + tot < fac) then
           tot = tot + mult
           i=i+1
           if (i< nend) mult = nint(coldata(1,i))
        else
           thin_ix(nout) = i
           nout= nout+1
           if (mult == fac - tot) then  
              i=i+1
              if (i< nend) mult = nint(coldata(1,i))
           else
              mult = mult - (fac -tot)
           end if
           tot = 0
        end if
        
     end do
       
     thin_rows = nout
     close(50)
   end subroutine ThinData

   subroutine GetCovMatrix
     integer i, j
     real mean(max_cols)
     real scale
     real, dimension(:,:), allocatable :: covmatrix
     
     allocate(corrmatrix(ncols-2,ncols-2))
     
     corrmatrix = 0
     
     do i=1, ncols-2
        if (isused(i+2)) then
           mean(i) = sum(coldata(1,0:nrows-1)*coldata(i+2,0:nrows-1))/numsamp
        end if
     end do
     
     do i = 1, ncols-2
        if (isused(i+2)) then
           do j = i, ncols-2
              if (isused(j+2)) then
                 corrmatrix(i,j) = sum(coldata(1,0:nrows-1)*((coldata(2+i,0:nrows-1)-mean(i))* &
                      (coldata(2+j,0:nrows-1)-mean(j))))/numsamp
                 corrmatrix(j,i) = corrmatrix(i,j)
              end if
           end do
        end if
     end do
     
     if (covmat_dimension /= 0) then
        write (*,*) 'Writing covariance matrix for ',covmat_dimension,' parameters'
        allocate(covmatrix(covmat_dimension,covmat_dimension))
        covmatrix=corrmatrix(1:covmat_dimension,1:covmat_dimension)
        call WriteSqMatrix(trim(outn)//trim(rootname) //'.covmat',covmatrix, covmat_dimension)
        deallocate(covmatrix)
     end if
     
     do i=1, ncols-2
        if (corrmatrix(i,i) > 0) then
           scale = sqrt(corrmatrix(i,i))
           corrmatrix(i,:) = corrmatrix(i,:)/scale
           corrmatrix(:,i) = corrmatrix(:,i)/scale
        end if
     end do
     call WriteSqMatrix(trim(outn)//trim(rootname) //'.corr',corrmatrix, ncols -2)
     
   end subroutine GetCovMatrix
   
   
   function MostCorrelated2D(i1,i2, direc)
     integer, intent(in) :: i1,i2, direc
     integer MostCorrelated2D
     !Find which parameter is most correllated with the degeneracy in ix1, ix2
     integer pars(2)
     real mat2D(2,2), evals(2), u(2,2)
     real corrs(2,ncols-2)
     
     if (direc /= 0 .and. direc /= -1) stop 'Invalid 3D color parameter'
     
     pars(1)=i1
     pars(2)=i2
     mat2D = corrmatrix(pars,pars)
     u = mat2D
     
     call Diagonalize(u, evals, 2)
     corrs = matmul(transpose(u), corrmatrix(pars,:))
     corrs(:,pars) = 0
     
     MostCorrelated2D = MaxIndex(abs(corrs(2+direc,colix(1:num_vars)-2)), num_vars)
     MostCorrelated2D = colix(MostCorrelated2D) -2
     
   end function MostCorrelated2D

   subroutine GetFractionIndices(fraction_indices,n)
     integer, intent(in) :: n
     integer num, fraction_indices(*), i
     real tot, aim
     
     tot = 0
     aim=numsamp/n
     num = 1
     fraction_indices(1) = 0
     fraction_indices(n+1) = nrows
     do i=0, nrows-1
        tot = tot + coldata(1,i)
        if (tot > aim) then
           num=num+1
           fraction_indices(num) = i
           if (num==n) exit              
           aim=aim+numsamp/n
        end if
     end do
     
   end subroutine GetFractionIndices
   
   function ConfidVal(ix,limfrac,upper,ix1,ix2)
     integer, intent(IN) :: ix
     real, intent(IN) :: limfrac
     logical, intent(IN) :: upper
     integer, intent(IN), optional :: ix1,ix2
     real ConfidVal, samps
     real try, lasttry, try_t, try_b
     integer l,t 
     !find upper and lower bounds
     
     l=0
     t=nrows-1
     if (present(ix1)) l=ix1
     if (present(ix2)) t = ix2
     try_b = minval(coldata(ix,l:t))
     try_t = maxval(coldata(ix,l:t))
     samps = sum(coldata(1,l:t))
     
     lasttry = -1
     if (upper) then
        do 
           try = sum(coldata(1,l:t),mask = coldata(ix,l:t) > (try_b + try_t)/2) 
           if (try > samps*limfrac) then
              try_b = (try_b+try_t)/2
           else
              try_t = (try_b+try_t)/2
           end if
           if (try == lasttry) exit
           lasttry = try
        end do
     else
        do 
           try = sum(coldata(1,l:t),mask = coldata(ix,l:t) < (try_b + try_t)/2) 
           if (try > samps*limfrac) then
              try_t = (try_b+try_t)/2
           else
              try_b = (try_b+try_t)/2
           end if
           if (try == lasttry) exit
           lasttry = try
        end do
     end if
     
     ConfidVal = try_t
   end function ConfidVal
   
   subroutine GetUsedCols
     integer j
     
     do j = 3, ncols
        isused(j) = any(coldata(j,0:nrows-1)/=coldata(j,0)) 
     end do
   end subroutine GetUsedCols
   
   subroutine DoConvergeTests
     real chain_means(max_chains,max_cols),chain_samp(max_chains)
     real between_chain_var(max_cols), in_chain_var(max_cols)
     integer frac(max_split_tests+1), split_n, chain_start(max_chains)
     integer i,j,k,jj,kk,ix
     real split_tests(max_split_tests), split_mean
     real mean(max_cols), fullmean(max_cols)
     real, parameter :: cutfrac = 0.5
     real usedsamps,evals(max_cols),sc,R, maxsamp
     real, dimension(:,:), allocatable :: cov, meanscov
     integer usedvars(max_cols), num, thin_fac(max_chains), markov_thin(max_chains)
     real(gp) u, g2, fitted, focus, alpha, beta, probsum, tmp1
     integer i1,i2,i3, nburn(max_chains), hardest, endb,hardestend
     integer tran(2,2,2), tran2(2,2)
     integer, dimension(:), allocatable :: binchain
     real, parameter :: epsilon = 0.001

     ! Get statistics for individual chains, and do split tests on the samples
     
     call CreateTxtFile(trim(outn)//trim(rootname) //'.converge', 40)
     
     if (num_chains_used > 1) write (*,*) 'Number of chains used =  ',num_chains_used
     
     chain_indices(num_chains_used+1) = nrows
     do i=1, num_chains_used
        chain_start(i) =  chain_indices(i) + (chain_indices(i+1)-chain_indices(i))*cutfrac 
        chain_samp(i) = sum(coldata(1,chain_start(i):chain_indices(i+1)-1)) 
     end do
     usedsamps = sum(chain_samp(1:num_chains_used))
     maxsamp = maxval(chain_samp(1:num_chains_used))
     num = 0  
     do j = 3, ncols
        if (isused(j)) then
            mean(j)=0
            fullmean(j) =  sum(coldata(1,0:nrows-1)*coldata(j,0:nrows-1))/numsamp
            
            if (num_chains_used> 1) then
               num=num+1
               usedvars(num)=j
               do i=1, num_chains_used
                  mean(j) = mean(j) + sum(coldata(1,chain_start(i):chain_indices(i+1)-1)*&
                       coldata(j,chain_start(i):chain_indices(i+1)-1))
               end do
               mean(j) = mean(j)/usedsamps
            end if
         end if
      end do
      
      if (num_chains_used > 1) then
         
         write (40,*) ''
         write(40,*)  'Variance test convergence stats using last half chains'
         write (40,*) 'param var(chain mean)/mean(chain var)'
         write (40,*) ''
         
         do j = 3, ncols
            between_chain_var(j) = 0
            in_chain_var(j) = 0
            
            if (isused(j)) then
               
               if (num_chains_used > 1) then
                  !Get stats for individual chains - the variance of the means over the mean of the variances
                  do i=1, num_chains_used
                     chain_means(i,j) =   sum(coldata(1,chain_start(i):chain_indices(i+1)-1)* &
                          coldata(j,chain_start(i):chain_indices(i+1)-1))/chain_samp(i) 
                     
                     between_chain_var(j) = between_chain_var(j) + &
                          ! chain_samp(i)/maxsamp* & !Weight for different length chains
                          (chain_means(i,j) - mean(j))**2
                     
                     in_chain_var(j) = in_chain_var(j) +  & !chain_samp(i)/maxsamp *&
                          sum(coldata(1,chain_start(i):chain_indices(i+1)-1)* &
                          (coldata(j,chain_start(i):chain_indices(i+1)-1)-chain_means(i,j))**2)
                     
                  end do
                  between_chain_var(j) = between_chain_var(j)/(num_chains_used-1) !(usedsamps/maxsamp -1)
                  in_chain_var(j) = in_chain_var(j)/usedsamps
                  write (40,'(1I3,f9.4,"  '//trim(lables(j))//'")') j-2, & 
                       between_chain_var(j) /in_chain_var(j)
               end if
               
            end if
         end do
      end if
     
      if (num_chains_used > 1 .and. covmat_dimension>0) then
         !Assess convergence in the var(mean)/mean(var) in the worst eigenvalue
         !c.f. Brooks and Gelman 1997          
         
         do while (usedvars(num) > covmat_dimension+2) 
            num= num - 1
         end do
         
         allocate(meanscov(num,num))
         allocate(cov(num,num))
         
         do jj=1,num
            j=usedvars(jj)
            do kk=jj, num
               k = usedvars(kk)
               meanscov(jj,kk) =0
               cov(jj,kk)= 0
               do i= 1, num_chains_used
                  
                  cov(jj,kk) = cov(jj,kk) + &
                       !sqrt(chain_samp(jj)*chain_samp(kk))/maxsamp * &
                       sum(coldata(1,chain_start(i):chain_indices(i+1)-1)* &
                       (coldata(j,chain_start(i):chain_indices(i+1)-1)-chain_means(i,j))* &
                       (coldata(k,chain_start(i):chain_indices(i+1)-1)-chain_means(i,k)))
                  
                  meanscov(jj,kk) = meanscov(jj,kk)+ & !sqrt(chain_samp(jj)*chain_samp(kk))/maxsamp * &
                       (chain_means(i,j)-mean(j))*(chain_means(i,k)-mean(k))
               end do
               meanscov(kk,jj) = meanscov(jj,kk)
               cov(kk,jj) = cov(jj,kk)              
               
            end do
         end do
         meanscov = meanscov/(num_chains_used-1) !(usedsamps/maxsamp -1)
         cov = cov / usedsamps
         
         do jj=1,num
            sc = sqrt(cov(jj,jj))
            cov(jj,:) = cov(jj,:) / sc
            cov(:,jj) = cov(:,jj) / sc
            meanscov(jj,:) = meanscov(jj,:) /sc
            meanscov(:,jj) = meanscov(:,jj) /sc                 
         end do
         
         call Diagonalize(meanscov, evals, num)
         cov = matmul(matmul(transpose(meanscov),cov),meanscov)
         write (40,*) ''
         write (40,*) 'var(mean)/mean(var) for eigenvalues of covariance of means'
         R = 0
         do jj=1,num
            write (40,'(1I3,f9.4)') jj,evals(jj)/cov(jj,jj)       
            R = max(R,evals(jj)/cov(jj,jj))
         end do
         !R is essentially the Gelman and Rubin statistic
         write (*,'(" var(mean)/mean(var), 1/2 chains, worst e-value: R-1 = ",f9.4)') R      
         
         deallocate(cov,meanscov)                 
      end if
      
      
      !Do tests for robustness under using splits of the samples
      !Return the rms ([difference from the mean]/[standard deviation]) when data split into 2, 3,.. sets
      write (40,*) ''
      write (40,*)  'Split tests: rms_n([delta mean]/sd) n={2,3,4,5}:'
      write(40,*) 'rms_n is the rms using the total samples split into n subsets'
      
      do j = 3, ncols
         if (isused(j)) then
            
            do split_n = 2,max_split_tests
               call GetFractionIndices(frac,split_n)
               split_tests(split_n) = 0
               do i=1,split_n
                  split_mean = sum(coldata(1,frac(i):frac(i+1)-1)*coldata(j,frac(i):frac(i+1)-1)) &
                       /sum(coldata(1,frac(i):frac(i+1)-1))
                  split_tests(split_n) = split_tests(split_n) + (split_mean - fullmean(j))**2 &
                       /( sum(coldata(1,frac(i):frac(i+1)-1)*(coldata(j,frac(i):frac(i+1)-1)-split_mean)**2)/ &
                       sum(coldata(1,frac(i):frac(i+1)-1)))            
               end do
               split_tests(split_n) = sqrt(split_tests(split_n)/split_n) !/split_var
            end do
            
            write (40,'(1I3,'//trim(IntToStr(max_split_tests-1)) // 'f9.4,"  ' &
                 //trim(lables(j))//'")') j-2, split_tests(2:max_split_tests)
         end if
      end do

      ! Now do Raftery and Lewis method
      ! See http://www.stat.washington.edu/tech.reports/raftery-lewis2.ps
      ! Raw non-importance sampled chains only
      
      if (.false.) then !jsut skip it 
         if (all(abs(coldata(1,0:nrows-1) - nint(coldata(1,0:nrows-1)))<1e-4)) then
            nburn = 0
            
            do ix=1, num_chains_used
               thin_fac(ix) = nint(maxval(coldata(1,chain_indices(ix):chain_indices(ix+1)-1)))
               
               do j = 3, covmat_dimension+2
                  if (isused(j) .and. (force_twotail .or. .not. has_limits(j))) then          
                     do endb =0,1
                        !Get binary chain depending on whether above or below confidence value
                        u = ConfidVal(j,(1-contours(num_contours))/2,endb==0,&
                             chain_indices(ix),chain_indices(ix+1)-1)
                        do !thin_fac
                           call ThinData(thin_fac(ix),chain_indices(ix),chain_indices(ix+1)-1)
                           if (thin_rows < 2) exit  
                           allocate(binchain(0:thin_rows-1))
                           where (coldata(j,thin_ix(0:thin_rows-1)) >= u)
                              binchain = 1
                           elsewhere
                              binchain = 2
                           endwhere
                           tran = 0
                           !Estimate transitions probabilities for 2nd order process
                           do i = 2, thin_rows-1
                              tran(binchain(i-2),binchain(i-1),binchain(i)) = &
                                   tran(binchain(i-2),binchain(i-1),binchain(i)) +1 
                           end do
                           deallocate(binchain)
                           
                           !Test whether 2nd order is better than Markov using BIC statistic
                           g2 = 0
                           do i1=1,2
                              do i2=1,2
                                 do i3=1,2
                                    if (tran(i1,i2,i3)/=0) then
                                       fitted = dble( (tran(i1,i2,1) + tran(i1,i2,2)) * &
                                            (tran(1,i2,i3) + tran(2,i2,i3)) ) / dble( tran(1,i2,1) + &
                                            tran(1,i2,2) + tran(2,i2,1) + tran(2,i2,2) )
                                       focus = dble( tran(i1,i2,i3) )
                                       g2 = g2 + log( focus / fitted ) * focus
                                       
                                    end if
                                 end do !i1
                              end do !i2
                           end do !i3
                           g2 = g2 * 2 
                           if (g2 - log( dble(thin_rows-2) ) * 2 < 0) exit
                           thin_fac(ix) = thin_fac(ix) + 1
                        end do !thin_fac
                        
                        !Get Markov transition probabilities for binary processes
                        if (sum(tran(:,1,2))==0 .or. sum(tran(:,2,1))==0) then
                           thin_fac(ix) = 0
                           goto 203
                        end if
                        alpha = sum(tran(:,1,2))/dble(sum(tran(:,1,1))+sum(tran(:,1,2)))
                        beta =  sum(tran(:,2,1))/dble(sum(tran(:,2,1))+sum(tran(:,2,2)))
                        probsum = alpha + beta
                        tmp1 = log(probsum * epsilon / max(alpha,beta))/ log( dabs(1.0d0 - probsum) )
                        if (int( tmp1 + 1 ) * thin_fac(ix) > nburn(ix)) then
                           nburn(ix) = int( tmp1 + 1 ) * thin_fac(ix)
                           hardest = j
                           hardestend = endb
                        end if
                     end do
                  end if
               end do !j
               
               markov_thin(ix) = thin_fac(ix)
               
               !Get thin factor to have independent samples rather than Markov
               
               u = ConfidVal(hardest,(1-contours(num_contours))/2,hardestend==0)
               thin_fac(ix) = thin_fac(ix) + 1             
               do !thin_fac
                  call ThinData(thin_fac(ix),chain_indices(ix),chain_indices(ix+1)-1)
                  if (thin_rows < 2) exit  
                  allocate(binchain(0:thin_rows-1))
                  where (coldata(hardest,thin_ix(0:thin_rows-1)) > u)
                     binchain = 1
                  elsewhere
                     binchain = 2
                  endwhere
                  tran2 = 0
                  !Estimate transitions probabilities for 2nd order process
                  do i = 1, thin_rows-1
                     tran2(binchain(i-1),binchain(i)) = &
                          tran2(binchain(i-1),binchain(i)) +1 
                  end do
                  deallocate(binchain)
                  
                  !Test whether independence is better than Markov using BIC statistic
                  g2 = 0
                  do i1=1,2
                     do i2=1,2
                        if (tran2(i1,i2)/=0) then
                           fitted = dble( (tran2(i1,1) + tran2(i1,2)) * (tran2(1,i2) + &
                                tran2(2,i2)) ) / dble(thin_rows -1)
                           focus = dble( tran2(i1,i2) )
                           g2 = g2 + dlog( focus / fitted ) * focus
                        end if
                     end do !i1
                  end do !i2
                  g2 = g2 * 2
                  
                  if (g2 - log( dble(thin_rows-1) ) < 0) exit
                  thin_fac(ix) = thin_fac(ix) + 1
               end do !thin_fac
               
203            if (thin_rows < 2) thin_fac(ix) = 0
               
            end do !chains
            
            write (40,*) ''
            write (40,*) 'Raftery&Lewis statistics'
            write (40,*) ''
            write (40,*) 'chain  markov_thin  indep_thin    nburn'
            do ix = 1, num_chains_used
               if (thin_fac(ix)==0) then
                  write (40,'(1I4,"      Not enough samples")') chain_numbers(ix)
               else
                  write (40,'(1I4,3I12)') chain_numbers(ix), markov_thin(ix), &
                       thin_fac(ix), nburn(ix)
               end if
            end do
            
            if (any(thin_fac(1:num_chains_used)==0)) then
               write (*,*) 'RL: Not enough samples to estimate convergence stats'
            else        
               call writeS ('RL: Thin for Markov:         '//&
                    Trim(IntToStr(maxval(markov_thin(1:num_chains_used)))))
               indep_thin = maxval(thin_fac(1:num_chains_used))
               call writeS('RL: Thin for indep samples:  '// &
                    trim(IntToStr(indep_thin)))            
               call WriteS('RL: Estimated burn in steps: '//&
                    trim(IntToStr(maxval(nburn(1:num_chains_used))))//' ('//&
                    trim(IntToStr(nint(maxval(nburn(1:num_chains_used))/mean_mult)))//' rows)')
               
            end if
            
         end if
         close(40)
      end if !skips this test, if .false.

    end subroutine DoConvergeTests

    subroutine Get2DPlotData(j,j2)
      integer, intent(in) :: j,j2
      integer i,ix1,ix2,wx,wy
      real binweight, dist, norm, maxbin
      real try_b, try_t,try_sum, try_last
      character(LEN=120) :: plotfile, filename,numstr
      
      bins2D = 0
      bin2Dlikes = 0
      bin2Dmax = 1e30        
      
      do i = 0, nrows-1
         ix1=nint((coldata(colix(j),i)-center(j))/width(j))
         ix2=nint((coldata(colix(j2),i)-center(j2))/width(j2))
         !puts all points above/below limits into upper/lowest bin
         if (ix1 > ix_max(j)) ix1 = ix_max(j)
         if (ix1 < ix_min(j)) ix1 = ix_min(j)
         if (ix2 > ix_max(j2)) ix2 = ix_max(j2)
         if (ix2 < ix_min(j2)) ix2 = ix_min(j2)
         
         if (smoothing) then
            do wx = max(ix_min(j),ix1-2),min(ix_max(j),ix1+2)
               dist = (coldata(colix(j),i) - (center(j) + wx*width(j)))**2/width(j)**2
               do wy = max(ix_min(j2),ix2-2),min(ix_max(j2),ix2+2)
                  binweight = coldata(1,i)*exp( - (&
                       dist +(coldata(colix(j2),i) - (center(j2) + wy*width(j2)))**2/width(j2)**2 )/2)
                  bins2D(wx,wy) = bins2D(wx,wy) + binweight
                  bin2Dlikes(wx,wy) = bin2Dlikes(wx,wy) + binweight*exp(meanlike - coldata(2,i))
               end do
            end do
         else
            bins2D(ix1,ix2) = bins2D(ix1,ix2) + coldata(1,i) 
            bin2Dlikes(ix1,ix2) = bin2Dlikes(ix1,ix2) + coldata(1,i)*exp(meanlike-coldata(2,i))
         end if
         bin2Dmax(ix1,ix2) = min(bin2Dmax(ix1,ix2),real(coldata(2,i)))
      end do
      
      do ix1=ix_min(j),ix_max(j)
         do ix2 =ix_min(j2),ix_max(j2)
            !take away bins with only a few samples
            !if (bins2D(ix1,ix2) < 0.00001*numsamp) then
            !   bins2D(ix1,ix2) = 0
            !   bin2Dlikes(ix1,ix2) = 0
            !end if
            if (bins2D(ix1,ix2) >0) bin2Dlikes(ix1,ix2) = bin2Dlikes(ix1,ix2)/bins2D(ix1,ix2)
            if (plot_NDcontours) then
               !Map maximum likelihood in each bin into a significance value from the full N-D distribution
               if (bin2DMax(ix1,ix2) == 1e30) then
                  bin2DMax(ix1,ix2) = 0
               else
                  bin2DMax(ix1,ix2) = sum(coldata(1,0:nrows-1), mask = coldata(2,0:nrows-1) > bin2DMax(ix1,ix2))/numsamp
               end if
            end if
         end do
      end do
      
      !if (has_limits(colix(j)) .or. has_limits(colix(j2))) then
      
      !Fix up underweighting near edges. Note this makes edge pixels noisier.
      ! do ix1 = ix_min(j), ix_max(j)
      !   do ix2 = ix_min(j2),ix_max(j2)
      !      if (ix1 ==ix_min(j) .and. has_limits_bot(colix(j))) bins2D(ix1,ix2) = bins2D(ix1,ix2)*2
      !      if (ix1 ==ix_max(j) .and. has_limits_top(colix(j))) bins2D(ix1,ix2) = bins2D(ix1,ix2)*2
      !      if (ix2 ==ix_min(j2).and. has_limits_bot(colix(j2))) bins2D(ix1,ix2) = bins2D(ix1,ix2)*2
      !      if (ix2 ==ix_max(j2).and. has_limits_top(colix(j2))) bins2D(ix1,ix2) = bins2D(ix1,ix2)*2
      !
      !     if (ix1 ==ix_min(j)+1.and. has_limits_bot(colix(j))) bins2D(ix1,ix2) = bins2D(ix1,ix2)/0.84
      !     if (ix1 ==ix_max(j)-1.and. has_limits_top(colix(j))) bins2D(ix1,ix2) = bins2D(ix1,ix2)/0.84
      !     if (ix2 ==ix_min(j2)+1.and. has_limits_bot(colix(j2))) bins2D(ix1,ix2) = bins2D(ix1,ix2)/0.84
      !     if (ix2 ==ix_max(j2)-1.and. has_limits_top(colix(j2))) bins2D(ix1,ix2) = bins2D(ix1,ix2)/0.84
      !  end do
      !end do
      
      !end if
      
      ! Get contour containing contours(:) of the probability
      
      allocate(TheBins(ix_max(j)-ix_min(j)+1-2*skip_bin,ix_max(j2)-ix_min(j2)+1-2*skip_bin))
      TheBins = bins2D(ix_min(j)+skip_bin:ix_max(j)-skip_bin,ix_min(j2)+skip_bin:ix_max(j2)-skip_bin)
      norm = sum(TheBins)
      
      do ix1 = 1, num_contours
         try_t = maxval(TheBins)
         try_b = 0
         try_last = -1
         do
            try_sum = sum(TheBins,mask = TheBins < (try_b + try_t)/2) 
            if (try_sum > (1-contours(ix1))*norm) then
               try_t = (try_b+try_t)/2
            else
               try_b = (try_b+try_t)/2
            end if
            if (try_sum == try_last) exit
            try_last = try_sum
         end do
         contour_levels(ix1) = (try_b+try_t)/2
         
      end do
      deallocate(TheBins)
      
      !Marginalized coloured shade
      plotfile = numcat(trim(numcat(trim(rootname)//'_2D_',colix(j)-2))//'_',colix(j2)-2)
      filename = 'plot_data/'//trim(plotfile)
      open(unit=49,file=filename,form='formatted',status='replace')
      do ix1 = ix_min(j)+skip_bin, ix_max(j)-skip_bin
         write (49,trim(numcat('(',ix_max(j2)-ix_min(j2)+1-2*skip_bin))//'E15.5)') bins2D(ix1,ix_min(j2)+skip_bin:ix_max(j2)-skip_bin)
      end do
      close(49)
      
      !MArginalized contours
      open(unit=49,file=trim(filename)//'_cont',form='formatted',status='replace')
      write(numstr,*) contour_levels(1:num_contours)
      if (num_contours==1) numstr = trim(numstr)//' '//trim(numstr)
      write (49,*) trim(numstr)
      close(49)
      
      !Mean like coloured shades
      open(unit=49,file=trim(filename)//'_likes',form='formatted',status='replace')
      maxbin = maxval(bin2Dlikes(ix_min(j)+skip_bin:ix_max(j)-skip_bin,ix_min(j2)+skip_bin:ix_max(j2)-skip_bin))
      do ix1 = ix_min(j)+skip_bin, ix_max(j)-skip_bin
         write (49,trim(numcat('(',ix_max(j2)-ix_min(j2)+1-2*skip_bin))//'E15.5)') &
              bin2Dlikes(ix1,ix_min(j2)+skip_bin:ix_max(j2)-skip_bin)/maxbin
      end do
      close(49)
      
      if (plot_NDcontours) then
         open(unit=49,file=trim(filename)//'_confid',form='formatted',status='replace')
         do ix1 = ix_min(j)+skip_bin, ix_max(j)-skip_bin
            write (49,trim(numcat('(',ix_max(j2)-ix_min(j2)+1-2*skip_bin))//'E15.5)') &
                 bin2Dmax(ix1,ix_min(j2)+skip_bin:ix_max(j2)-skip_bin)
            
         end do
         close(49)
      end if
      
    end subroutine Get2DPlotData


 
end module MCSamples

