!module to perform post-processing of chains
!useful to add data/recompute predictions/add new observables
!SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
!This version May 2007 
module postprocess

  use calclike

  implicit none

  contains 

  subroutine PostProcChains(in_unit)
    integer, intent(IN) :: in_unit
    !outfile_unit is already open and globally defined
    integer :: ierr, line, rejected,l
    real :: mult, like, NewLike
    real :: Xt, Xb, mu, tan_2thetat, tan_2thetab
    character (LEN=100) :: InLine 
    Type(ParamSet) :: P
    Type(Input_params) :: ChainMSSM
    Type(Nuisance_params) :: ChainNuisP
    Type(DS_Output) :: ChainDSP
    Type(Susy_spec) :: ChainSusySp
    Type(SoftOut) :: ChainSoftOut
    Type(Bdecay_Out) :: ChainBdecay
    Type(Error_Out) :: ChainSoftErr
    
    
    ChainSoftOut%neut%mneut = LogZero
    
    line = 0
    rejected = 0
    if (skip_lines > 0) then
       if (Feedback > 1) write(*,*) 'Skipping ', skip_lines, ' lines'
       do l=1,skip_lines
          read(in_unit, *, iostat=ierr) mult
          if (ierr < 0) call DoStop('Skipped to many lines while post-processing!')
       end do
    end if

    
    do !loops over lines in file 
       call SetFormat !this is in paramdef.f90
       read (in_unit, fmt_params, ADVANCE='NO', END = 77) mult, like, P%P
       !reading in the params is enough, you have to recompute the rest anyways
       if (redo_like .and. .not. redo_theory) then
          !you want to read in the previous values and simply adjust the like
          !the flags must match the chain or this will go horribly wrong
          if (GFlags%DM_predict)  read(in_unit, fmt_dm, ADVANCE='NO') ChainDSP%dm
          if (GFlags%DD_predict)  read(in_unit, fmt_dd, ADVANCE='NO') ChainDSP%dd
          if (GFlags%Collider_predict)  read(in_unit, fmt_coll, ADVANCE='NO') ChainDSP%coll
          if (GFlags%BD_predict)  read(in_unit, fmt_bdecay, ADVANCE='NO') ChainBdecay
          read(in_unit, fmt_susysp, ADVANCE='NO')  ChainSusySp
          read(in_unit, fmt_xtxbmu, ADVANCE='NO')  Xt, Xb, mu
          read(in_unit, fmt_tantheta, ADVANCE='NO')  tan_2thetat, tan_2thetab
          if (GFlags%FH_predict)  read(in_unit, fmt_prodxbr, ADVANCE='NO') ChainReducedProdxbr

       end if

       !This advances to the next line 
       read(in_unit, '(A)',ADVANCE='YES',EOR=66, END = 77) InLine
66     continue

      !need to convert back some of the quantities
      !write(*,*) 'before conversion', P%P
      call ConvertFromReading(P,ChainDSP) 
      

      line = line + 1
      if (Feedback > 1) write(*,*) 'reading in line ', line
      if (Feedback > 2) write(*,*)  'params: ', P%P

      if (redo_theory) then
         !recomputes everything from the basic params
         NewLike = GetLogLike(P)
      else
         !only recomputes like
         call ParamsToMSSMParams(P%P, ChainMSSM, ChainNuisP)
         !unfortunately we still have to call SoftSusy to compute the spectrum 
         !(needed to compute the Higgs like)
         call GetSoftSusySpectrum(ChainMSSM, ChainNuisP, ChainSusySp, ChainSoftOut, ChainSoftErr)
         NewLike = GetLogLikePost(ChainSusySp, ChainNuisP, ChainDSP, ChainMSSM, ChainSoftOut, ChainBDecay)
         !those have NOT been recomputed, setting them equal to the chain's values
         CurrentOutputP = ChainDSP
         CurrentBdecay  = ChainBDecay
      end if
      !since WriteParams actually writes the Previous param set
      !we need to copy the Current into Previous
      !the Current contains the values which go with P
      PreviousOutputP = CurrentOutputP
      PreviousSusySp  = CurrentSusySp
      PreviousSoftOut = CurrentSoftOut
      !The Feynhiggs predictions in CurrentProdxbr are empty if redo_theory = .false.
      !deal with this as a special case in WriteParams
      PreviousProdxbr = CurrentProdxbr
      PreviousBdecay  = CurrentBdecay


      if (redo_like) then
         !adjust like and multiplicity
         if (Feedback > 0) then
            write(*,*) ' ---- Changes in Like and Multiplicity  -----'
            write(*,'(A,F20.5,A,F20.5)') 'NewLike  = ', Like, ' DeltaLike = ', Like-NewLike
            write(*,'(A,F20.5,A,F20.5)') 'New mult = ', mult*exp(like-NewLike), ' Old mult  = ', mult
            if (.not. redo_theory) then
               write(*,*) ' ---- Changes in some of the observable values  -----'
               write(*,'(A,F10.6)') 'Percent Omega_CDM = ',  1d0 -  PreviousOutputP%dm%oh2/ChainDSP%dm%oh2
               write(*,'(A,F10.6)') 'Percent g-2       = ',  1d0 - (PreviousOutputP%coll%gm2amp*1E10)/(ChainDSP%coll%gm2amp*1E10) 
               write(*,'(A,F10.6)') 'Percent Bsgamma   = ',  1d0 - (PreviousOutputP%coll%bsgamma*1E4)/(ChainDSP%coll%bsgamma*1E4) 
               write(*,'(A,F10.6)') 'Percent M_W       = ',  1d0 - (PreviousOutputP%coll%mW)/(ChainDSP%coll%mW)
               write(*,'(A,F10.6)') 'Percent sin_eff   = ',  1d0 - (PreviousOutputP%coll%sweff2)/(ChainDSP%coll%sweff2)
            end if
         end if
         mult = mult*exp(like-NewLike)
         like = NewLike 
      end if
      if (like < LogZero) then
         !write the point only if its allowed under the new like
         call WriteParams(P, mult, like)
      else
         rejected = rejected + 1
      end if
   end do

77 continue

   write(*,*)  'Points rejected under new like:', rejected 
   write(*,*)  'Chain ', instance, ': post-processed models: ', line

 end subroutine PostProcChains

end module postprocess
